import os
from datetime import datetime
from pathlib import Path
from typing import Any, Literal

from pydantic import BaseModel, Field

from .logger import bootstrap_logger

LOGGER_LEVEL = os.getenv("LOGGER_LEVEL", "INFO")
logger = bootstrap_logger(level=LOGGER_LEVEL)


class StepRecord(BaseModel):
    """Record detailed information of task execution steps"""

    step_name: str
    message: str
    timestamp: datetime
    status: Literal["info", "warning", "failed", "success", "debug"] = "info"
    metadata: dict[str, Any] = Field(default_factory=dict)


class TaskTracer(BaseModel):
    """Only use primitive types, datatime, Path etc."""

    status: Literal["pending", "running", "completed", "interrupted", "failed"] = "pending"

    # task info. hydrated BEFORE task execution.
    task_id: str = ""
    task_name: str = ""
    task_file_name: str | None = ""
    ground_truth: str | None
    input: Any = None

    # not task-related info. hydrated BEFORE task execution.
    log_path: Path

    # profile exeuction time. hydrated AFTER task execution.
    start_time: datetime = Field(default_factory=datetime.now)
    end_time: datetime = Field(default_factory=datetime.now)

    # record task result. hydrdrated AFTER task execution.
    final_boxed_answer: str = ""
    judge_result: str = ""
    error: str = ""

    # record task exection detail. hydrated DURING task_execution.
    current_main_turn_id: int = 0
    current_sub_agent_turn_id: int = 0
    sub_agent_counter: int = 0
    current_sub_agent_session_id: str | None = None
    main_agent_message_history: dict[str, Any] = Field(default_factory=dict)
    sub_agent_message_history_sessions: dict[str, dict[str, Any]] = Field(default_factory=dict)
    step_logs: list[StepRecord] = Field(default_factory=list)

    def start_sub_agent_session(self, sub_agent_name: str, subtask_description: str) -> str:
        """Start a new sub-agent session"""
        self.sub_agent_counter += 1
        session_id = f"{sub_agent_name}_{self.sub_agent_counter}"
        self.current_sub_agent_session_id = session_id

        # record sub-agent session start
        self.log_step(
            f"sub_{sub_agent_name}_session_start",
            f"Starting {session_id} for subtask: {subtask_description[:100]}{'...' if len(subtask_description) > 100 else ''}",
            "info",
            metadata={"session_id": session_id, "subtask": subtask_description},
        )

        return session_id

    def end_sub_agent_session(self, sub_agent_name: str):
        """End the current sub-agent session"""
        self.log_step(
            f"sub_{sub_agent_name}_session_end",
            f"Ending {self.current_sub_agent_session_id}",
            "success",
            metadata={"session_id": self.current_sub_agent_session_id},
        )
        self.current_sub_agent_session_id = None
        return None

    def log_step(
        self,
        step_name: str,
        message: str,
        status: Literal["info", "warning", "failed", "success", "debug"] = "info",
        metadata: dict[str, Any] | None = None,
    ):
        """Record execution step"""
        step_log = StepRecord(
            step_name=step_name,
            message=message,
            timestamp=datetime.now(),
            status=status,
            metadata=metadata or {},
        )
        self.step_logs.append(step_log)
        # Also print to console
        logger.debug(f"{step_name}: {message}")

    def save(self):
        """Persist TaskTracer to disk. used in a finally block, thus never raise Exception."""
        try:
            if not self.log_path.exists():
                self.log_path.parent.mkdir(exist_ok=True, parents=True)
            with open(self.log_path, mode="w") as dest:
                dest.write(self.model_dump_json(indent=2))
        except Exception as e:
            logger.error(e, stack_info=True, exc_info=True)

    def cleanup(self):
        """
        Clean up memory-heavy fields after task execution.
        Call this after save() to release memory.
        """
        try:
            # Clear message histories (can be large with many turns)
            self.main_agent_message_history.clear()
            self.sub_agent_message_history_sessions.clear()

            # Clear step logs (can accumulate many entries)
            self.step_logs.clear()

            # Clear input data
            self.input = None

            logger.debug(f"TaskTracer cleanup completed for task_id={self.task_id}")
        except Exception as e:
            logger.warning(f"Error during TaskTracer cleanup: {e}")



"""
Orchestrator 模块

协调 Agent 执行流程的核心模块，负责：
- 主 Agent 执行循环
- 工具调用协调
- 子 Agent 调度
- 流式输出管理
"""

import asyncio
import os
import re
import uuid
from collections.abc import Callable
from typing import Any

from omegaconf import DictConfig

from mem_deep_research_core.core.answer_handler import post_process_final_answer
from mem_deep_research_core.core.context_manager import ContextManager, ContextManagerConfig
from mem_deep_research_core.core.hooks import HookContext, hooks
from mem_deep_research_core.core.interceptor_config import InterceptorConfig, InterceptorPresets
from mem_deep_research_core.core.llm_call_handler import (
    LLMCallHandler,
    SummaryHandler,
)
from mem_deep_research_core.core.main_loop import MainLoopRunner
from mem_deep_research_core.core.message_interceptor import MessageInterceptorHandler
from mem_deep_research_core.core.monitoring import (
    ExecutionMonitor,
    MonitoringConfig,
)

# 导入拆分后的模块
from mem_deep_research_core.core.stream_handler import StreamHandler
from mem_deep_research_core.core.sub_agent_runner import SubAgentRunner
from mem_deep_research_core.core.task_planner import TaskPlanner
from mem_deep_research_core.core.tool_executor import ToolExecutor
from mem_deep_research_core.core.tool_result_formatter import ToolResultFormatter
from mem_deep_research_core.core.user_context import UserContextBuilder, detect_language_by_chars
from mem_deep_research_core.llm.provider_client_base import LLMProviderClientBase
from mem_deep_research_core.mem_deep_research_logging.logger import (
    bootstrap_logger,
)
from mem_deep_research_core.mem_deep_research_logging.task_tracer import TaskTracer
from mem_deep_research_core.prompts.template_loader import PromptTemplateLoader
from mem_deep_research_core.tool.manager import ToolManager
from mem_deep_research_core.utils.external_loader import external_loader
from mem_deep_research_core.utils.io_utils import OutputFormatter, process_input
from mem_deep_research_core.utils.stream_parsing_utils import TextInterceptor
from mem_deep_research_core.utils.summary_utils import (
    detect_response_language,
    extract_hints,
)
from mem_deep_research_core.utils.tool_utils import _load_agent_prompt, expose_sub_agents_as_tools

# ========== 默认钩子实现 ==========


def _default_on_agent_start(ctx: HookContext):
    """Agent 开始执行 - 默认实现"""
    logger.debug(
        f"[Hook] on_agent_start: query={ctx.query}, agent_type={ctx.extra.get('agent_type')}"
    )
    return None


def _default_on_agent_end(ctx: HookContext):
    """Agent 执行完成 - 默认实现"""
    logger.debug(f"[Hook] on_agent_end: query={ctx.query}, turns_used={ctx.turn_number}")
    return None


def _default_on_turn_start(ctx: HookContext):
    """轮次开始 - 默认实现"""
    logger.debug(f"[Hook] on_turn_start: turn={ctx.turn_number}")
    return None


def _default_on_turn_end(ctx: HookContext):
    """轮次结束 - 默认实现"""
    logger.debug(
        f"[Hook] on_turn_end: turn={ctx.turn_number}, tool_calls_count={ctx.tool_calls_count}"
    )
    return None


hooks.set_default("on_agent_start", _default_on_agent_start)
hooks.set_default("on_agent_end", _default_on_agent_end)
hooks.set_default("on_turn_start", _default_on_turn_start)
hooks.set_default("on_turn_end", _default_on_turn_end)

LOGGER_LEVEL = os.getenv("LOGGER_LEVEL", "INFO")
logger = bootstrap_logger(level=LOGGER_LEVEL)


def _list_tools(sub_agent_tool_managers: dict[str, ToolManager]):
    """创建带缓存的工具列表获取函数"""
    cache = None

    async def wrapped():
        nonlocal cache
        if cache is None:
            if not sub_agent_tool_managers:
                result = {}
            else:
                result = {
                    name: await tool_manager.get_all_tool_definitions()
                    for name, tool_manager in sub_agent_tool_managers.items()
                }
            cache = result
        return cache

    return wrapped


def _generate_message_id() -> str:
    """生成随机消息 ID"""
    return f"msg_{uuid.uuid4().hex[:8]}"


class Orchestrator:
    """Agent 执行协调器"""

    def __init__(
        self,
        main_agent_tool_manager: ToolManager,
        sub_agent_tool_managers: dict[str, ToolManager],
        llm_client: LLMProviderClientBase,
        output_formatter: OutputFormatter,
        cfg: DictConfig,
        task_log: TaskTracer,
        sub_agent_llm_client: LLMProviderClientBase | None = None,
        stream_queue: Any | None = None,
        tool_definitions: list[dict[str, Any]] | None = None,
        sub_agent_tool_definitions: dict[str, list[dict[str, Any]]] | None = None,
        context: dict[str, Any] | None = None,
    ):
        # 基础组件
        self.main_agent_tool_manager = main_agent_tool_manager
        self.sub_agent_tool_managers = sub_agent_tool_managers
        self.llm_client = llm_client
        self.sub_agent_llm_client = sub_agent_llm_client or llm_client
        self.output_formatter = output_formatter
        self.cfg = cfg
        self.task_log = task_log
        self.stream_queue = stream_queue
        self.tool_definitions = tool_definitions
        self.sub_agent_tool_definitions = sub_agent_tool_definitions
        self.context = context or {}

        # 模板加载器
        self._template_loader = PromptTemplateLoader()

        # 缓存函数
        self._list_sub_agent_tools = _list_tools(sub_agent_tool_managers)

        # 配置解析
        chinese_context_val = self.cfg.main_agent.get("chinese_context", False)
        if isinstance(chinese_context_val, str):
            self.chinese_context = chinese_context_val.lower().strip() == "true"
        else:
            self.chinese_context = bool(chinese_context_val)

        add_message_id_val = self.cfg.main_agent.get("add_message_id", False)
        if isinstance(add_message_id_val, str):
            self.add_message_id: bool = add_message_id_val.lower().strip() == "true"
        else:
            self.add_message_id: bool = bool(add_message_id_val)
        logger.info(f"add_message_id: {self.add_message_id}")

        # 目标语言
        self.target_language: str | None = None
        self.target_language_task: asyncio.Task | None = None

        # 传递 task_log 给 LLM 客户端
        if self.llm_client and task_log:
            self.llm_client.task_log = task_log
        if self.sub_agent_llm_client and task_log and self.sub_agent_llm_client != self.llm_client:
            self.sub_agent_llm_client.task_log = task_log

        # 设置上下文到工具管理器
        if self.context and self.context.get("user_id"):
            self.main_agent_tool_manager.set_context(self.context)
            for sub_manager in self.sub_agent_tool_managers.values():
                sub_manager.set_context(self.context)

        # ========== 初始化组合模块 ==========
        self._init_modules()

    def _init_modules(self):
        """初始化各个组合模块"""
        # 流式处理器
        self.stream_handler = StreamHandler(self.stream_queue)

        # 工具结果格式化器
        self.tool_formatter = ToolResultFormatter(self.context)

        # 用户上下文构建器
        self.context_builder = UserContextBuilder(self.context, self.chinese_context)

        # 消息拦截处理器
        interceptor_config = self._load_interceptor_config()
        self.key_message_interceptor = TextInterceptor(
            interceptor_config.get_all_filter_keywords(),
            reasoning_tags=interceptor_config.reasoning_tags,
        )
        self.message_interceptor = MessageInterceptorHandler(
            config=interceptor_config,
            stream_reasoning_callback=self.stream_handler.stream_reasoning,
            stream_tool_call_callback=self.stream_handler.stream_tool_call,
            stream_message_callback=self.stream_handler.stream_message,
            context=self.context,
        )

        # 执行监控器（从配置读取）
        monitoring_cfg_dict = self.cfg.main_agent.get("monitoring", {})
        if monitoring_cfg_dict and not isinstance(monitoring_cfg_dict, dict):
            monitoring_cfg_dict = (
                dict(monitoring_cfg_dict) if hasattr(monitoring_cfg_dict, "__iter__") else {}
            )
        if monitoring_cfg_dict:
            from mem_deep_research_core.config_schema import MonitoringConfigSchema

            monitoring_schema = MonitoringConfigSchema(**monitoring_cfg_dict)
            monitoring_config = MonitoringConfig.from_schema(monitoring_schema)
        else:
            monitoring_config = MonitoringConfig()
        self._monitoring_schema_dict = monitoring_cfg_dict or {}
        self.monitor = ExecutionMonitor(
            config=monitoring_config,
            stream_reasoning_callback=self._stream_tool_reasoning,  # 保留：有自定义逻辑
        )

        # 工具执行器
        self.tool_executor = ToolExecutor(
            tool_manager=self.main_agent_tool_manager,
            output_formatter=self.output_formatter,
            tool_result_formatter=self.tool_formatter,
            context=self.context,
            stream_tool_call=self.stream_handler.stream_tool_call,
            stream_tool_reasoning=self._stream_tool_reasoning,  # 保留：有自定义逻辑
            stream_usage_info=self.stream_handler.stream_usage_info,
        )
        # scrape_max_length: 优先从配置读，fallback 环境变量，最后默认 20000
        scrape_max_length = self._monitoring_schema_dict.get("scrape_max_length", None)
        if scrape_max_length is None:
            try:
                scrape_max_length = int(os.getenv("SCRAPE_MAX_LENGTH", "20000"))
            except (ValueError, TypeError):
                scrape_max_length = 20000
        self.tool_executor.set_scrape_max_length(scrape_max_length)

        # 子 Agent 运行器
        self.sub_agent_runner = SubAgentRunner(
            sub_agent_tool_managers=self.sub_agent_tool_managers,
            sub_agent_llm_client=self.sub_agent_llm_client,
            output_formatter=self.output_formatter,
            cfg=self.cfg,
            task_log=self.task_log,
            context=self.context,
            chinese_context=self.chinese_context,
            stream_start_agent=self.stream_handler.stream_start_agent,
            stream_end_agent=self.stream_handler.stream_end_agent,
            stream_start_llm=self.stream_handler.stream_start_llm,
            stream_end_llm=self.stream_handler.stream_end_llm,
            stream_tool_call=self.stream_handler.stream_tool_call,
            stream_tool_reasoning=self._stream_tool_reasoning,  # 保留：有自定义逻辑
            stream_usage_info=self.stream_handler.stream_usage_info,
            handle_llm_call=self._handle_llm_call_with_logging,
            handle_summary=self._handle_summary_with_context_limit_retry,
            intercept_key_message=self._intercept_key_message,
        )
        if self.sub_agent_tool_definitions:
            self.sub_agent_runner.set_cached_tool_definitions(self.sub_agent_tool_definitions)

        # LLM 调用处理器
        self.llm_handler = LLMCallHandler(
            main_llm_client=self.llm_client,
            sub_agent_llm_client=self.sub_agent_llm_client,
            task_log=self.task_log,
            add_message_id=self.add_message_id,
            keep_tool_result=self.cfg.main_agent.keep_tool_result,
            stream_error_callback=self.stream_handler.stream_tool_call,
        )

        # 摘要处理器
        self.summary_handler = SummaryHandler(
            llm_call_handler=self.llm_handler,
            chinese_context=self.chinese_context,
        )

        # Task Planner — 仅在 deep_research.enabled AND auto_planning 时启用
        dr_cfg = self.cfg.main_agent.get("deep_research", {})
        planning_enabled = (
            dr_cfg and dr_cfg.get("enabled", False) and dr_cfg.get("auto_planning", False)
        )
        self.task_planner = TaskPlanner(enabled=planning_enabled)

        # Inline Skill Selector
        self.inline_skill_selector = external_loader.get_inline_skill_selector(
            self.cfg, chinese=self.chinese_context
        )
        # 将 <next_skills> 注册为 reasoning tag，使其被 TextInterceptor 自动提取并从输出中剥离
        if self.inline_skill_selector:
            from mem_deep_research_core.skills.inline_selector import InlineSkillSelector

            current_tags = list(self.key_message_interceptor.tag_extractor.reasoning_tags)
            if InlineSkillSelector.TAG_NAME not in current_tags:
                current_tags.append(InlineSkillSelector.TAG_NAME)
                self.key_message_interceptor.set_reasoning_tags(current_tags)

        # Context Manager (三级 context 管理 + dedup + source registry)
        cm_cfg_dict = self.cfg.main_agent.get("context_manager", {})
        if cm_cfg_dict and not isinstance(cm_cfg_dict, dict):
            cm_cfg_dict = dict(cm_cfg_dict) if hasattr(cm_cfg_dict, "__iter__") else {}
        cm_config = ContextManagerConfig(**cm_cfg_dict) if cm_cfg_dict else ContextManagerConfig()
        self.context_manager = ContextManager(config=cm_config)
        # 注入 token 估算函数
        if hasattr(self.llm_client, "_estimate_tokens"):
            self.context_manager.set_token_estimator(self.llm_client._estimate_tokens)

        # 当前 Agent ID（用于流式输出）
        self.current_agent_id: str | None = None

    @staticmethod
    def _extract_recent_tool_names(message_history: list, lookback: int = 6) -> list:
        """从最近消息中提取 tool_use 的 name 列表"""
        names = []
        for msg in message_history[-lookback:]:
            content = msg.get("content")
            if isinstance(content, list):
                for block in content:
                    if isinstance(block, dict) and block.get("type") == "tool_use":
                        name = block.get("name")
                        if name and name not in names:
                            names.append(name)
        return names

    @staticmethod
    def _deduplicate_trailing_messages(message_history: list) -> int:
        """移除消息历史末尾重复的 assistant 响应，保留第一次出现。

        当循环检测终止时，message_history 可能包含多轮相同的 assistant 响应，
        这会导致摘要 LLM 困惑或生成空内容。此方法从末尾向前扫描，
        移除连续重复的 assistant 消息（基于文本内容 hash），
        并在末尾追加一条说明，引导摘要 LLM 基于已有信息作答。

        Returns:
            int: 移除的消息数量
        """
        if len(message_history) < 4:
            return 0

        def _text_hash(msg: dict) -> int:
            content = msg.get("content", "")
            if isinstance(content, list):
                texts = [
                    item.get("text", "")
                    for item in content
                    if isinstance(item, dict) and item.get("type") == "text"
                ]
                text = "".join(texts)
            elif isinstance(content, str):
                text = content
            else:
                text = str(content)
            return hash(text[:500]) if text else 0

        # 从末尾收集连续 assistant 消息的 hash
        i = len(message_history) - 1
        tail_hashes = []
        while i >= 0 and message_history[i].get("role") == "assistant":
            tail_hashes.append((i, _text_hash(message_history[i])))
            i -= 1
            # 跳过中间的 user 消息（如 INJECT_HINT）
            if i >= 0 and message_history[i].get("role") == "user":
                content_str = str(message_history[i].get("content", ""))
                if "MANDATORY DIRECTIVE" in content_str or "SYSTEM WARNING" in content_str:
                    i -= 1

        if len(tail_hashes) < 2:
            return 0

        # 找出重复 hash 的索引，保留最早的一个
        seen_hashes = {}
        indices_to_remove = []
        for idx, h in reversed(tail_hashes):  # 从前往后遍历
            if h in seen_hashes:
                indices_to_remove.append(idx)
            else:
                seen_hashes[h] = idx

        if not indices_to_remove:
            return 0

        # 同时移除紧跟在被删 assistant 消息后面的 INJECT_HINT user 消息
        all_remove = set(indices_to_remove)
        for idx in indices_to_remove:
            # 检查 idx+1 和 idx-1 是否为注入的指令消息
            for neighbor in (idx + 1, idx - 1):
                if 0 <= neighbor < len(message_history) and neighbor not in all_remove:
                    msg = message_history[neighbor]
                    if msg.get("role") == "user":
                        content_str = str(msg.get("content", ""))
                        if "MANDATORY DIRECTIVE" in content_str or "SYSTEM WARNING" in content_str:
                            all_remove.add(neighbor)

        # 按索引从大到小移除
        for idx in sorted(all_remove, reverse=True):
            if idx < len(message_history):
                message_history.pop(idx)

        removed = len(all_remove)
        if removed > 0:
            # 追加引导消息，帮助摘要 LLM 生成有效输出
            message_history.append(
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": (
                                "[SYSTEM] The research process was terminated due to a repeated response loop. "
                                "Please synthesize a final answer based on all the information gathered so far. "
                                "If insufficient information was collected, acknowledge the limitation and provide "
                                "the best possible answer with what is available."
                            ),
                        }
                    ],
                }
            )
            logger.info(
                f"[DEDUP] Removed {removed} duplicate/injected messages from history tail, "
                f"history now {len(message_history)} messages"
            )

        return removed

    def _load_interceptor_config(self) -> InterceptorConfig:
        """从配置文件加载拦截器配置"""
        interceptor_cfg = self.cfg.main_agent.get("interceptor", {})

        if not interceptor_cfg:
            return InterceptorConfig()

        preset = interceptor_cfg.get("preset")
        if preset:
            config = InterceptorPresets.from_name(preset)
            logger.info(f"[INTERCEPTOR] Using preset: {preset}")
        else:
            config = InterceptorConfig()

        # 应用自定义配置
        if "filter_tags" in interceptor_cfg:
            config.filter_tags = list(interceptor_cfg.filter_tags)
        if "reasoning_tags" in interceptor_cfg:
            config.reasoning_tags = list(interceptor_cfg.reasoning_tags)
        if "show_reasoning" in interceptor_cfg:
            config.show_reasoning = interceptor_cfg.show_reasoning
        if "show_tool_calls" in interceptor_cfg:
            config.show_tool_calls = interceptor_cfg.show_tool_calls
        if "show_text_output" in interceptor_cfg:
            config.show_text_output = interceptor_cfg.show_text_output
        if "strip_reasoning_from_output" in interceptor_cfg:
            config.strip_reasoning_from_output = interceptor_cfg.strip_reasoning_from_output
        if "custom_tag_handlers" in interceptor_cfg:
            config.custom_tag_handlers = dict(interceptor_cfg.custom_tag_handlers)

        logger.info(
            f"[INTERCEPTOR] Config: filter_tags={config.filter_tags}, reasoning_tags={config.reasoning_tags}"
        )
        return config

    # ========== 流式输出方法 ==========

    async def _stream_tool_reasoning(
        self, tool_name: str, action: str, details: str, parent_uid: str = None
    ):
        """输出工具调用的推理过程"""
        reasoning_id = f"tool_reason_{uuid.uuid4().hex[:8]}"

        # START / RESULT_SUMMARY 的 details 由 hooks 生成，已是完整格式化内容，直接使用。
        # 其他 action（ERROR、监控事件等）保留框架前缀。
        if action in ("START", "RESULT_SUMMARY", "QUERY"):
            content = details
        elif action == "ERROR":
            content = f"❌ **工具错误**: {tool_name}\n{details}"
        else:
            content = f"📝 **{action}**: {details}"

        await self.stream_handler.stream_reasoning(
            reasoning_id=reasoning_id,
            content=content,
            parent_uid=parent_uid or self.current_agent_id,
            status="SUCCESS",
        )

    # ========== 用户上下文方法 ==========

    def _build_user_identity_context(self) -> str:
        return self.context_builder.build_user_identity_context()

    # ========== 消息拦截方法 ==========

    async def _intercept_key_message(self, message_id: str, message: str, is_last: bool):
        """拦截并处理流式消息"""
        try:
            result, reasoning_blocks = self.key_message_interceptor.process(message, is_last)

            if reasoning_blocks:
                logger.info(f"[REASONING] Extracted {len(reasoning_blocks)} blocks")

            for block in reasoning_blocks:
                try:
                    await self.stream_handler.stream_reasoning(
                        reasoning_id=block.uid,
                        content=block.content,
                        parent_uid=self.current_agent_id,
                        status="SUCCESS",
                    )
                except Exception as e:
                    logger.warning(f"Failed to stream reasoning block: {e}")

            if result is not None:
                if not result.strip():
                    return True
                if self.key_message_interceptor.is_unbreakable_string(result):
                    return False
                else:
                    await self.stream_handler.stream_tool_call(
                        "show_text", {"text": result}, True, message_id
                    )
                    await asyncio.sleep(0)
                    return True
            return True
        except Exception as e:
            logger.error(f"Error in _intercept_key_message: {e}")
            try:
                if message and message.strip():
                    await self.stream_handler.stream_tool_call(
                        "show_text", {"text": message}, True, message_id
                    )
            except Exception:
                pass
            return True

    async def _streaming_final_message(self, message_id: str, message: str, is_last: bool):
        """最终消息的流式输出"""
        try:
            if not hasattr(self, "_final_message_interceptor"):
                self._final_message_interceptor = TextInterceptor(["<use_mcp_tool>"])

            result, reasoning_blocks = self._final_message_interceptor.process(message, is_last)

            for block in reasoning_blocks:
                try:
                    await self.stream_handler.stream_reasoning(
                        reasoning_id=block.uid,
                        content=block.content,
                        parent_uid=self.current_agent_id,
                        status="SUCCESS",
                    )
                except Exception as e:
                    logger.warning(f"Failed to stream final reasoning block: {e}")

            if result is not None:
                if not result.strip():
                    return True
                if self._final_message_interceptor.is_unbreakable_string(result):
                    return False
                else:
                    await self.stream_handler.stream_message(
                        message_id=message_id, delta_content=result
                    )
                    await asyncio.sleep(0)
                    return True
            return True
        except Exception as e:
            logger.error(f"Error in _streaming_final_message: {e}")
            try:
                if message and message.strip():
                    await self.stream_handler.stream_message(
                        message_id=message_id, delta_content=message
                    )
            except Exception:
                pass
            return True

    # ========== 语言检测 ==========

    async def _detect_language(self, text: str) -> str:
        """检测响应语言"""
        try:
            api_key = self.cfg.main_agent.get("openai_api_key")
            base_url = os.environ.get("OPENAI_BASE_URL", "https://api.openai.com/v1")
            if not api_key:
                return detect_language_by_chars(text)

            return await detect_response_language(
                query=text,
                api_key=api_key,
                base_url=base_url,
            )
        except Exception as e:
            logger.warning(f"Language detection failed: {e}")
            return detect_language_by_chars(text)

    # ========== Context 压缩 LLM 调用 ==========

    async def _context_summarize_call(
        self,
        summarize_system_prompt: str,
        summarize_messages: list,
        purpose: str,
    ) -> str:
        """Level 2 context 压缩的 LLM 调用"""
        response_text, _, _ = await self.llm_call_handler.handle_llm_call(
            summarize_system_prompt,
            summarize_messages,
            [],  # 不需要工具
            999,
            purpose,
            agent_type="main",
        )
        return response_text or ""

    # ========== LLM 调用处理 ==========

    async def _handle_llm_call_with_logging(
        self,
        system_prompt,
        message_history,
        tool_definitions,
        step_id: int,
        purpose: str = "LLM call",
        keep_tool_result: int = -1,
        agent_type: str = "main",
        stream_message_callback: Callable = None,
    ) -> tuple[str | None, bool, Any | None]:
        """统一的 LLM 调用和日志处理，委托给 LLMCallHandler"""
        return await self.llm_handler.handle_llm_call(
            system_prompt=system_prompt,
            message_history=message_history,
            tool_definitions=tool_definitions,
            step_id=step_id,
            purpose=purpose,
            agent_type=agent_type,
            stream_message_callback=stream_message_callback,
        )

    async def _handle_summary_with_context_limit_retry(
        self,
        system_prompt,
        agent_prompt_instance,
        message_history,
        tool_definitions,
        purpose,
        task_description,
        task_failed,
        agent_type="main",
        task_guidence="",
        stream_message_callback: Callable = None,
        deep_research_cfg: dict = None,
    ):
        """处理摘要生成，委托给 SummaryHandler"""
        self.summary_handler.target_language = self.target_language
        self.summary_handler.target_language_task = self.target_language_task

        return await self.summary_handler.handle_summary_with_retry(
            system_prompt=system_prompt,
            agent_prompt_instance=agent_prompt_instance,
            message_history=message_history,
            tool_definitions=tool_definitions,
            purpose=purpose,
            task_description=task_description,
            task_failed=task_failed,
            agent_type=agent_type,
            task_guidance=task_guidence,
            stream_message_callback=stream_message_callback,
            detect_language_func=self._detect_language,
        )

    # ========== 子 Agent 运行 ==========

    async def run_sub_agent(self, sub_agent_name, task_description, keep_tool_result: int = -1):
        """运行子 Agent"""
        return await self.sub_agent_runner.run(
            sub_agent_name=sub_agent_name,
            task_description=task_description,
            keep_tool_result=keep_tool_result,
        )

    # ========== 主 Agent 运行 ==========

    async def run_main_agent(
        self, task_description, task_file_name=None, task_id="default_task", history=None
    ):
        """执行主 Agent 任务"""
        workflow_id = await self.stream_handler.stream_start_workflow(task_description, task_id)
        keep_tool_result = int(self.cfg.main_agent.keep_tool_result)

        logger.debug(f"\n{'=' * 20} Starting Task: {task_id} {'=' * 20}")
        logger.debug(f"Task Description: {task_description}")

        # 1. 处理输入
        initial_user_content, task_description = process_input(task_description, task_file_name)
        task_guidence = self._build_task_guidance()
        initial_user_content[0]["text"] = initial_user_content[0]["text"] + task_guidence

        # 2. 生成提示词（如果启用）
        hint_notes = await self._generate_hints(task_description)
        if hint_notes:
            initial_user_content[0]["text"] = initial_user_content[0]["text"] + hint_notes

        logger.info("Initial user input content: %s", initial_user_content)

        # 3. 构建消息历史
        message_history = self._build_initial_history(history)
        message_history.append({"role": "user", "content": initial_user_content})

        # 4. 获取工具定义
        tool_definitions = await self._get_tool_definitions()
        self.task_log.log_step("get_main_tool_definitions", f"{tool_definitions}")

        # 4.5. LLM Skill 选择
        selected_skill_names = await self._select_skills(initial_user_content, tool_definitions)

        # 5. 生成系统提示词
        system_prompt, main_agent_prompt_instance, deep_research_cfg = self._build_system_prompt(
            tool_definitions, initial_user_content, selected_skill_names
        )

        # 6. 主循环
        final_answer_text = await self._run_main_loop(
            system_prompt=system_prompt,
            message_history=message_history,
            tool_definitions=tool_definitions,
            main_agent_prompt_instance=main_agent_prompt_instance,
            deep_research_cfg=deep_research_cfg,
            task_description=task_description,
            task_guidence=task_guidence,
            keep_tool_result=keep_tool_result,
        )

        # 7. 后处理
        final_summary, final_boxed_answer = await post_process_final_answer(
            cfg=self.cfg,
            final_answer_text=final_answer_text,
            task_description=task_description,
            message_history=message_history,
            system_prompt=system_prompt,
            chinese_context=self.chinese_context,
            task_log=self.task_log,
            output_formatter=self.output_formatter,
            llm_client=self.llm_client,
        )

        # 结束流式输出
        await self.stream_handler.stream_end_llm("reporter")
        await self.stream_handler.stream_end_agent("reporter", self.current_agent_id)

        main_agent_usage = self.llm_client.get_usage()
        await self.stream_handler.stream_usage_info("main", main_agent_usage, "main_agent_end")

        if self.sub_agent_llm_client and self.sub_agent_llm_client is not self.llm_client:
            sub_agent_usage = self.sub_agent_llm_client.get_usage()
            await self.stream_handler.stream_usage_info(
                "sub_agent", sub_agent_usage, "sub_agent_end"
            )

        await self.stream_handler.stream_end_workflow(workflow_id)

        logger.debug(f"\n{'=' * 20} Task {task_id} Finished {'=' * 20}")
        self.task_log.log_step("task_completed", f"Task {task_id} completed successfully")

        if "browsecomp-zh" in self.cfg.benchmark.name:
            return final_summary, final_summary
        else:
            return final_summary, final_boxed_answer

    def _build_task_guidance(self) -> str:
        """构建任务指导"""
        if self.chinese_context:
            return self._template_loader.load_template("guidance/task_guidance_chinese")
        return ""

    async def _generate_hints(self, task_description: str) -> str:
        """生成任务提示"""
        if not self.cfg.main_agent.input_process.hint_generation:
            return ""

        try:
            hint_content = await extract_hints(
                task_description,
                self.cfg.main_agent.openai_api_key,
                self.chinese_context,
                self.add_message_id,
                self.cfg.main_agent.input_process.get(
                    "hint_llm_base_url", "https://api.openai.com/v1"
                ),
            )
            return self._template_loader.load_and_render(
                "hints/hint_prefix",
                hint_content=hint_content,
            )
        except Exception as e:
            logger.error(f"Hint generation failed: {str(e)}")
            self.task_log.log_step(
                "hint_generation", f"[ERROR] Hint generation failed: {str(e)}", "failed"
            )
            return ""

    def _build_initial_history(self, history) -> list:
        """构建初始消息历史"""
        message_history = []
        if history:
            for turn_history in history:
                for message in turn_history["main_agent"]:
                    content = message["content"]
                    if isinstance(content, str):
                        content = re.sub(
                            r"<think>.*?</think>", "", content, flags=re.DOTALL
                        ).strip()
                    message_history.append({"role": message["role"], "content": content})
        return message_history

    async def _get_tool_definitions(self) -> list:
        """获取工具定义"""
        if not self.tool_definitions:
            tool_definitions = await self.main_agent_tool_manager.get_all_tool_definitions()
            if self.cfg.sub_agents is not None and self.cfg.sub_agents:
                tool_definitions += expose_sub_agents_as_tools(self.cfg.sub_agents)
        else:
            tool_definitions = self.tool_definitions

        if not tool_definitions:
            logger.debug("Warning: No tool definitions found.")
        return tool_definitions

    async def _select_skills(
        self,
        query,
        tool_definitions: list,
    ) -> list[str] | None:
        """使用 LLM 选择相关 Skills

        inline 模式下跳过此步骤（第一轮用规则匹配，后续轮次从回复中解析）。

        Args:
            query: 用户查询（str 或 list）
            tool_definitions: 工具定义列表

        Returns:
            选中的 skill 名称列表，如果 LLM selector 不可用则返回 None
        """
        # inline 模式：不做额外 LLM 调用，第一轮用规则匹配 fallback
        if self.inline_skill_selector:
            return None

        try:
            selector = external_loader.get_llm_skill_selector(self.cfg)
            if not selector:
                return None

            tool_names = [t.get("name", "") for t in tool_definitions if isinstance(t, dict)]
            selected = await selector.select(
                query=query,
                tool_names=tool_names,
                context=self.context,
            )
            logger.info(f"LLM selected skills: {selected}")
            return selected
        except Exception as e:
            logger.warning(f"Skill selection failed: {e}")
            return None

    def _build_system_prompt(
        self, tool_definitions, initial_user_content, selected_skill_names=None
    ):
        """构建系统提示词"""
        # 获取 prompt 配置
        prompt_cfg = {}
        if hasattr(self.cfg.main_agent, "prompt") and self.cfg.main_agent.prompt:
            prompt_cfg = dict(self.cfg.main_agent.prompt)

        # deep_research 配置转换为 presets
        deep_research_cfg = getattr(self.cfg.main_agent, "deep_research", None)
        if deep_research_cfg is not None:
            deep_research_cfg = dict(deep_research_cfg)

        main_agent_prompt_instance = _load_agent_prompt(prompt_cfg)

        extra_context = self._build_user_identity_context()

        system_prompt = main_agent_prompt_instance.generate_system_prompt_with_mcp_tools(
            mcp_servers=tool_definitions,
            chinese_context=self.chinese_context,
            extra_context=extra_context,
            deep_research_cfg=deep_research_cfg,
        )

        # 注入 Skills
        if self.inline_skill_selector:
            # Inline 模式：追加 skill catalog 到 system prompt（告诉 LLM 可用 skill 列表）
            catalog_prompt = self.inline_skill_selector.build_skill_catalog_prompt()
            if catalog_prompt:
                system_prompt += catalog_prompt
                logger.debug("Inline skill catalog appended to system prompt")
        else:
            skill_injector = external_loader.get_skill_injector()
            if skill_injector and initial_user_content:
                if selected_skill_names is not None:
                    # LLM 选择路径：按名称直接注入
                    system_prompt = skill_injector.inject_selected_skills(
                        base_prompt=system_prompt,
                        skill_names=selected_skill_names,
                    )
                    logger.debug(f"LLM-selected skills injected: {selected_skill_names}")
                else:
                    # Fallback 路径：规则匹配注入
                    tools_to_use = [
                        t.get("name", "") for t in tool_definitions if isinstance(t, dict)
                    ]
                    system_prompt = skill_injector.inject_skills(
                        base_prompt=system_prompt,
                        query=initial_user_content,
                        context=self.context,
                        tools_to_use=tools_to_use,
                    )
                    logger.debug("Rule-matched skills injected into system prompt")

        return system_prompt, main_agent_prompt_instance, deep_research_cfg

    def _create_main_loop_runner(self) -> MainLoopRunner:
        """创建主循环运行器"""
        return MainLoopRunner(
            cfg=self.cfg,
            monitor=self.monitor,
            context_manager=self.context_manager,
            stream_handler=self.stream_handler,
            tool_executor=self.tool_executor,
            sub_agent_runner=self.sub_agent_runner,
            llm_handler=self.llm_handler,
            summary_handler=self.summary_handler,
            task_planner=self.task_planner,
            inline_skill_selector=self.inline_skill_selector,
            llm_client=self.llm_client,
            output_formatter=self.output_formatter,
            task_log=self.task_log,
            context=self.context,
            chinese_context=self.chinese_context,
            monitoring_schema_dict=self._monitoring_schema_dict,
            handle_llm_call=self._handle_llm_call_with_logging,
            handle_summary=self._handle_summary_with_context_limit_retry,
            intercept_key_message=self._intercept_key_message,
            streaming_final_message=self._streaming_final_message,
            stream_tool_reasoning=self._stream_tool_reasoning,
            extract_recent_tool_names=self._extract_recent_tool_names,
            deduplicate_trailing_messages=self._deduplicate_trailing_messages,
            build_user_identity_context=self._build_user_identity_context,
            detect_language=self._detect_language,
        )

    async def _run_main_loop(
        self,
        system_prompt,
        message_history,
        tool_definitions,
        main_agent_prompt_instance,
        deep_research_cfg,
        task_description,
        task_guidence,
        keep_tool_result,
    ):
        """运行主执行循环（委托给 MainLoopRunner）"""
        runner = self._create_main_loop_runner()
        result = await runner.run(
            system_prompt=system_prompt,
            message_history=message_history,
            tool_definitions=tool_definitions,
            main_agent_prompt_instance=main_agent_prompt_instance,
            deep_research_cfg=deep_research_cfg,
            task_description=task_description,
            task_guidence=task_guidence,
            keep_tool_result=keep_tool_result,
        )
        # 同步 current_agent_id（runner 内部会更新）
        self.current_agent_id = runner.current_agent_id
        return result



"""
最终答案处理模块

负责最终答案的后处理、提取和格式化输出。
从 orchestrator.py 中拆分出来以降低复杂度。
"""

import logging
from typing import Any

from mem_deep_research_core.mem_deep_research_logging.logger import truncate_for_log
from mem_deep_research_core.mem_deep_research_logging.task_tracer import TaskTracer
from mem_deep_research_core.utils.io_utils import OutputFormatter
from mem_deep_research_core.utils.summary_utils import (
    extract_browsecomp_zh_final_answer,
    extract_gaia_final_answer,
)

logger = logging.getLogger("mem_deep_research")


async def extract_final_answer(
    cfg,
    task_description: str,
    final_answer_text: str,
    message_history: list,
    chinese_context: bool,
    task_log: TaskTracer,
) -> str:
    """提取最终答案

    Args:
        cfg: Agent 配置
        task_description: 任务描述
        final_answer_text: 原始最终答案文本
        message_history: 消息历史
        chinese_context: 是否中文上下文
        task_log: 任务日志

    Returns:
        处理后的最终答案文本
    """
    try:
        final_answer_model = cfg.main_agent.output_process.get(
            "final_answer_model", "anthropic/claude-sonnet-4.5"
        )
        final_answer_base_url = cfg.main_agent.output_process.get(
            "final_answer_llm_base_url", "https://openrouter.ai/api/v1"
        )

        if "browsecomp-zh" in cfg.benchmark.name:
            extracted_answer = await extract_browsecomp_zh_final_answer(
                task_description,
                final_answer_text,
                cfg.main_agent.openai_api_key,
                final_answer_base_url,
                model=final_answer_model,
            )
            message_history.append(
                {
                    "role": "assistant",
                    "content": [
                        {"type": "text", "text": f"LLM extracted final answer:\n{extracted_answer}"}
                    ],
                }
            )
            return extracted_answer
        else:
            extracted_answer = await extract_gaia_final_answer(
                task_description,
                final_answer_text,
                cfg.main_agent.openai_api_key,
                chinese_context,
                final_answer_base_url,
                model=final_answer_model,
            )
            message_history.append(
                {
                    "role": "assistant",
                    "content": [
                        {"type": "text", "text": f"LLM extracted final answer:\n{extracted_answer}"}
                    ],
                }
            )
            return f"{final_answer_text}\n\nLLM Extracted Answer:\n{extracted_answer}"

    except Exception as e:
        logger.error(f"Final answer extraction failed: {str(e)}")
        task_log.log_step("final_answer_extraction", f"[ERROR] Failed: {str(e)}", "failed")
        return final_answer_text


async def post_process_final_answer(
    cfg,
    final_answer_text: str,
    task_description: str,
    message_history: list,
    system_prompt: str,
    chinese_context: bool,
    task_log: TaskTracer,
    output_formatter: OutputFormatter,
    llm_client: Any,
) -> tuple[str, str]:
    """后处理最终答案

    Args:
        cfg: Agent 配置
        final_answer_text: 原始最终答案
        task_description: 任务描述
        message_history: 消息历史
        system_prompt: 系统提示词
        chinese_context: 是否中文上下文
        task_log: 任务日志
        output_formatter: 输出格式化器
        llm_client: LLM 客户端

    Returns:
        (final_summary, final_boxed_answer) 格式化后的摘要和提取的答案
    """
    if final_answer_text:
        task_log.log_step("final_answer", "Final answer extracted successfully")
        task_log.log_step("final_answer_content", f"Content: {final_answer_text}")

        # 提取最终答案（如果启用）
        if cfg.main_agent.output_process.final_answer_extraction:
            final_answer_text = await extract_final_answer(
                cfg,
                task_description,
                final_answer_text,
                message_history,
                chinese_context,
                task_log,
            )
    else:
        final_answer_text = "No final answer generated."
        task_log.log_step("final_answer", "Failed to extract final answer", "failed")

    logger.debug(f"LLM Final Answer: {truncate_for_log(final_answer_text)}")

    # 保存最终消息历史
    task_log.main_agent_message_history = {
        "system_prompt": system_prompt,
        "message_history": message_history,
    }
    task_log.save()

    # 格式化输出
    task_log.log_step("format_output", "Formatting final output")
    final_summary, final_boxed_answer = output_formatter.format_final_summary_and_log(
        final_answer_text, llm_client
    )

    return final_summary, final_boxed_answer



"""
简单计算器 MCP 工具示例

演示如何创建自定义 MCP 工具供框架调用。
"""

import math

from fastmcp import FastMCP

# 初始化 MCP 服务器
mcp = FastMCP("calculator-server")


@mcp.tool()
async def calculate(expression: str) -> str:
    """执行数学计算表达式。

    支持基本运算 (+, -, *, /, **) 和数学函数 (sqrt, sin, cos, log 等)。

    Args:
        expression: 数学表达式，如 "2 + 3 * 4" 或 "sqrt(16) + log(100)"

    Returns:
        计算结果
    """
    try:
        # 安全的数学函数白名单
        safe_dict = {
            "sqrt": math.sqrt,
            "sin": math.sin,
            "cos": math.cos,
            "tan": math.tan,
            "log": math.log10,
            "ln": math.log,
            "exp": math.exp,
            "abs": abs,
            "pow": pow,
            "pi": math.pi,
            "e": math.e,
        }

        # 计算表达式
        result = eval(expression, {"__builtins__": {}}, safe_dict)
        return f"计算结果: {expression} = {result}"
    except Exception as e:
        return f"[ERROR]: 计算失败 - {str(e)}"


@mcp.tool()
async def unit_convert(value: float, from_unit: str, to_unit: str) -> str:
    """单位转换工具。

    Args:
        value: 要转换的数值
        from_unit: 原单位 (km, m, cm, mm, mile, ft, inch, kg, g, lb, oz, c, f, k)
        to_unit: 目标单位

    Returns:
        转换结果
    """
    # 长度单位（以米为基准）
    length_units = {
        "km": 1000,
        "m": 1,
        "cm": 0.01,
        "mm": 0.001,
        "mile": 1609.344,
        "ft": 0.3048,
        "inch": 0.0254,
    }

    # 重量单位（以克为基准）
    weight_units = {"kg": 1000, "g": 1, "mg": 0.001, "lb": 453.592, "oz": 28.3495}

    from_unit = from_unit.lower()
    to_unit = to_unit.lower()

    try:
        # 温度转换
        if from_unit in ["c", "f", "k"] and to_unit in ["c", "f", "k"]:
            if from_unit == "c":
                celsius = value
            elif from_unit == "f":
                celsius = (value - 32) * 5 / 9
            else:  # k
                celsius = value - 273.15

            if to_unit == "c":
                result = celsius
            elif to_unit == "f":
                result = celsius * 9 / 5 + 32
            else:  # k
                result = celsius + 273.15

            return f"{value} {from_unit.upper()} = {result:.2f} {to_unit.upper()}"

        # 长度转换
        if from_unit in length_units and to_unit in length_units:
            meters = value * length_units[from_unit]
            result = meters / length_units[to_unit]
            return f"{value} {from_unit} = {result:.4f} {to_unit}"

        # 重量转换
        if from_unit in weight_units and to_unit in weight_units:
            grams = value * weight_units[from_unit]
            result = grams / weight_units[to_unit]
            return f"{value} {from_unit} = {result:.4f} {to_unit}"

        return f"[ERROR]: 不支持的单位转换: {from_unit} -> {to_unit}"
    except Exception as e:
        return f"[ERROR]: 转换失败 - {str(e)}"


if __name__ == "__main__":
    mcp.run(transport="stdio", show_banner=False)



"""
用户上下文构建模块

处理用户身份信息、对话历史等上下文的构建。
支持 _secure 字段：敏感数据在 system prompt 中显示为占位符。
"""

from typing import Any

from mem_deep_research_core.core.secure_context import build_secure_usage_prompt, get_display_value


class UserContextBuilder:
    """用户上下文构建器"""

    def __init__(self, context: dict[str, Any] | None = None, chinese_context: bool = False):
        """
        初始化用户上下文构建器

        Args:
            context: 用户上下文字典，包含 user_id, user_name, org_id, timezone, mode 等
            chinese_context: 是否使用中文上下文
        """
        self.context = context or {}
        self.chinese_context = chinese_context

    def build_user_identity_context(self) -> str:
        """
        Build user identity context string to inject into the system prompt.

        This allows the LLM to know who the current user is when answering questions
        like "我是谁" (Who am I?).

        For mirror mode: Direct identity statement (e.g., "你是 XXX")
        For default mode: Service-oriented context (e.g., "你正在为 XXX 提供服务")

        Also includes conversation history (meta_chat_history) when provided.

        Returns:
            A formatted string with user identity information, or empty string if no context.
        """
        if not self.context:
            return ""

        # _secure 字段自动显示为占位符 [SECURE:xxx]，非 _secure 字段显示真实值
        user_id = get_display_value(self.context, "user_id")
        user_name = get_display_value(self.context, "user_name")
        org_id = get_display_value(self.context, "org_id")
        timezone = get_display_value(self.context, "timezone", "UTC")
        mode = self.context.get("mode", "default")
        meta_chat_history = self.context.get("meta_chat_history")

        if not user_id:
            return ""

        # Mirror mode: Direct identity - the AI IS the user
        if mode == "mirror":
            if self.chinese_context:
                if user_name:
                    identity_str = (
                        f"你是 **{user_name}**。当别人问你是谁时，直接说「我是{user_name}」。\n"
                    )
                else:
                    identity_str = f"你的用户ID是 {user_id}。\n"
            else:
                if user_name:
                    identity_str = f'You are **{user_name}**. When asked who you are, simply say "I\'m {user_name}".\n'
                else:
                    identity_str = f"Your user ID is {user_id}.\n"

            # Add role purpose context for mirror mode (auto-fetched)
            role_purpose = self.context.get("role_purpose")
            if role_purpose:
                if self.chinese_context:
                    identity_str += f"\n## 你的角色与职责\n\n{role_purpose}\n"
                else:
                    identity_str += f"\n## Your Role & Responsibilities\n\n{role_purpose}\n"

            # Add conversation history for mirror mode
            history_str = self._build_chat_history_context(meta_chat_history)
            if history_str:
                identity_str += "\n" + history_str

            # Append secure placeholder usage guide
            identity_str += build_secure_usage_prompt(self.context, chinese=self.chinese_context)

            return identity_str

        # Default mode: Service-oriented context
        if self.chinese_context:
            identity_parts = ["## 当前用户信息\n"]
            identity_parts.append("你正在为以下用户提供服务：\n")
            if user_name:
                identity_parts.append(f"- **用户名称**: {user_name}\n")
            identity_parts.append(f"- **用户ID**: {user_id}\n")
            if org_id:
                identity_parts.append(f"- **组织ID**: {org_id}\n")
            identity_parts.append(f"- **时区**: {timezone}\n")
            identity_parts.append(
                "\n当用户询问「我是谁」或类似身份相关问题时，你可以使用上述信息回答。"
            )
            identity_parts.append("你还可以使用 search_contacts 等工具查询该用户的更多详细信息。\n")
        else:
            identity_parts = ["## Current User Information\n"]
            identity_parts.append("You are serving the following user:\n")
            if user_name:
                identity_parts.append(f"- **User Name**: {user_name}\n")
            identity_parts.append(f"- **User ID**: {user_id}\n")
            if org_id:
                identity_parts.append(f"- **Organization ID**: {org_id}\n")
            identity_parts.append(f"- **Timezone**: {timezone}\n")
            identity_parts.append(
                "\nWhen the user asks 'Who am I?' or similar identity questions, you can use the above information to answer."
            )
            identity_parts.append(
                "You can also use tools like search_contacts to get more detailed information about this user.\n"
            )

        # Add conversation history for default mode
        history_str = self._build_chat_history_context(meta_chat_history)
        if history_str:
            identity_parts.append("\n" + history_str)

        # Append secure placeholder usage guide
        secure_prompt = build_secure_usage_prompt(self.context, chinese=self.chinese_context)
        if secure_prompt:
            identity_parts.append(secure_prompt)

        return "".join(identity_parts)

    def _build_chat_history_context(self, meta_chat_history: list) -> str:
        """
        Build conversation history context string to inject into the system prompt.

        This provides the LLM with context from previous conversation rounds.

        Args:
            meta_chat_history: List of chat history items with role, content, name fields.
                Format: [{"role": "user/assistant", "content": "...", "name": "..."}]

        Returns:
            A formatted string with conversation history, or empty string if no history.
        """
        if not meta_chat_history or len(meta_chat_history) == 0:
            return ""

        if self.chinese_context:
            history_parts = ["## 对话历史\n"]
            history_parts.append("以下是之前的对话记录，请参考这些上下文来理解当前问题：\n\n")
        else:
            history_parts = ["## Conversation History\n"]
            history_parts.append(
                "The following is the previous conversation history. Please refer to this context to understand the current question:\n\n"
            )

        for item in meta_chat_history:
            # Support both dict and object (MetaChatHistoryItem) formats
            if hasattr(item, "role"):
                # Object format (MetaChatHistoryItem)
                role = item.role or "user"
                content = item.content or ""
                name = getattr(item, "name", None)
            else:
                # Dict format
                role = item.get("role", "user")
                content = item.get("content", "")
                name = item.get("name")

            if role == "user":
                if self.chinese_context:
                    if name:
                        history_parts.append(f"**{name}**: {content}\n\n")
                    else:
                        history_parts.append(f"**用户**: {content}\n\n")
                else:
                    if name:
                        history_parts.append(f"**{name}**: {content}\n\n")
                    else:
                        history_parts.append(f"**User**: {content}\n\n")
            elif role == "assistant":
                if self.chinese_context:
                    history_parts.append(f"**助手**: {content}\n\n")
                else:
                    history_parts.append(f"**Assistant**: {content}\n\n")
            elif role == "system":
                if self.chinese_context:
                    history_parts.append(f"**系统**: {content}\n\n")
                else:
                    history_parts.append(f"**System**: {content}\n\n")

        history_parts.append("---\n\n")

        return "".join(history_parts)

    def get_prompt_class_for_mode(self, mode: str, default_prompt_class: str) -> str:
        """
        Get the appropriate prompt class name based on the operation mode.

        This allows different modes (e.g., 'mirror' vs 'default') to use different
        system prompts for specialized behavior.

        Args:
            mode: The operation mode from context ("default", "mirror", etc.)
            default_prompt_class: The default prompt class from config

        Returns:
            The prompt class name to use.
        """
        # Mode to prompt class mapping
        mode_prompt_mapping = {
            "mirror": "MainAgentPrompt_Mirror",  # Mirror chat uses specialized prompt
            "default": default_prompt_class,  # Default uses config value
        }

        # Return the mapped prompt class, or fall back to config default
        return mode_prompt_mapping.get(mode, default_prompt_class)


def detect_language_by_chars(text: str) -> str:
    """Fallback: detect language by character analysis."""
    chinese_chars = sum(1 for c in text if "\u4e00" <= c <= "\u9fff")
    japanese_chars = sum(1 for c in text if "\u3040" <= c <= "\u309f" or "\u30a0" <= c <= "\u30ff")
    korean_chars = sum(1 for c in text if "\uac00" <= c <= "\ud7af" or "\u1100" <= c <= "\u11ff")

    total_len = len(text)
    if total_len == 0:
        return "English"

    # Check ratios (threshold: 5%)
    if japanese_chars / total_len > 0.05:
        return "Japanese"
    if korean_chars / total_len > 0.05:
        return "Korean"
    if chinese_chars / total_len > 0.05:
        return "Chinese"

    return "English"



"""
主执行循环模块

从 Orchestrator 拆分，负责 Agent 主循环的执行逻辑：
- 轮次循环控制
- 工具调用执行与去重
- 监控检查与升级
- 反思检查点注入
- 最终摘要生成
"""

import logging
import sys
from collections.abc import Callable

from mem_deep_research_core.core.hooks import HookContext, hooks
from mem_deep_research_core.core.monitoring import (
    EscalationAction,
    TurnCounter,
)

logger = logging.getLogger("mem_deep_research")


class MainLoopRunner:
    """主执行循环运行器

    封装 Agent 主循环的完整执行逻辑，从 Orchestrator 拆分出来以降低复杂度。
    通过依赖注入获取所需组件，不直接引用 Orchestrator。
    """

    def __init__(
        self,
        *,
        cfg,
        monitor,
        context_manager,
        stream_handler,
        tool_executor,
        sub_agent_runner,
        llm_handler,
        summary_handler,
        task_planner,
        inline_skill_selector,
        llm_client,
        output_formatter,
        task_log,
        context: dict,
        chinese_context: bool,
        monitoring_schema_dict: dict,
        # callbacks
        handle_llm_call: Callable,
        handle_summary: Callable,
        intercept_key_message: Callable,
        streaming_final_message: Callable,
        stream_tool_reasoning: Callable,
        extract_recent_tool_names: Callable,
        deduplicate_trailing_messages: Callable,
        build_user_identity_context: Callable,
        detect_language: Callable,
    ):
        self.cfg = cfg
        self.monitor = monitor
        self.context_manager = context_manager
        self.stream_handler = stream_handler
        self.tool_executor = tool_executor
        self.sub_agent_runner = sub_agent_runner
        self.llm_handler = llm_handler
        self.summary_handler = summary_handler
        self.task_planner = task_planner
        self.inline_skill_selector = inline_skill_selector
        self.llm_client = llm_client
        self.output_formatter = output_formatter
        self.task_log = task_log
        self.context = context
        self.chinese_context = chinese_context
        self._monitoring_schema_dict = monitoring_schema_dict

        # Callbacks (injected from Orchestrator)
        self._handle_llm_call = handle_llm_call
        self._handle_summary = handle_summary
        self._intercept_key_message = intercept_key_message
        self._streaming_final_message = streaming_final_message
        self._stream_tool_reasoning = stream_tool_reasoning
        self._extract_recent_tool_names = extract_recent_tool_names
        self._deduplicate_trailing_messages = deduplicate_trailing_messages
        self._build_user_identity_context = build_user_identity_context
        self._detect_language = detect_language

        # 当前 Agent ID
        self.current_agent_id: str | None = None

    async def run(
        self,
        system_prompt: str,
        message_history: list,
        tool_definitions: list,
        main_agent_prompt_instance,
        deep_research_cfg: dict | None,
        task_description: str,
        task_guidence: str,
        keep_tool_result: int,
    ) -> str:
        """运行主执行循环

        Returns:
            最终答案文本
        """
        max_turns = self.cfg.main_agent.max_turns
        if max_turns < 0:
            max_turns = sys.maxsize
        max_tool_calls = self.cfg.main_agent.max_tool_calls_per_turn

        # 初始化监控和计数器
        self.monitor.reset()
        self.context_manager.reset()
        turn_counter = TurnCounter(
            max_turns=max_turns,
            reflection_enabled=deep_research_cfg and deep_research_cfg.get("enabled", False),
            reflection_interval=deep_research_cfg.get("reflection_interval", 5)
            if deep_research_cfg
            else 5,
        )

        task_failed = False
        self.current_agent_id = await self.stream_handler.stream_start_agent("main")
        await self.stream_handler.stream_start_llm("main")

        # Hook: on_agent_start
        hooks.call(
            "on_agent_start",
            HookContext(
                hook_name="on_agent_start",
                query=task_description,
                context=self.context,
                extra={"agent_type": "main"},
            ),
        )

        # 自动任务分解（仅深度研究模式 + auto_planning 启用时）
        if self.task_planner.enabled:
            plan = await self.task_planner.create_plan(
                task_description=task_description,
                llm_client=self.llm_client,
            )
            if plan:
                message_history.append(
                    {
                        "role": "user",
                        "content": [
                            {"type": "text", "text": f"[RESEARCH PLAN]\n{plan.to_context_string()}"}
                        ],
                    }
                )
                self.task_log.log_step(
                    "auto_planning",
                    f"Generated research plan with {len(plan.sub_questions)} sub-questions",
                )
                logger.info("[TaskPlanner] Plan injected into message history")

        while not turn_counter.is_max_reached():
            turn_count = turn_counter.increment()
            self.context_manager.set_turn(turn_count)
            logger.debug(f"\n--- Main Agent Turn {turn_count} ---")

            # Hook: on_turn_start
            hooks.call(
                "on_turn_start",
                HookContext(
                    hook_name="on_turn_start",
                    turn_number=turn_count,
                    query=task_description,
                    context=self.context,
                ),
            )
            self.task_log.save()

            # 监控前检查
            terminate_reason = await self.monitor.pre_turn_check()
            if terminate_reason:
                break

            # LLM 调用
            assistant_response_text, should_break, tool_calls = await self._handle_llm_call(
                system_prompt,
                message_history,
                tool_definitions,
                turn_count,
                f"Main agent turn {turn_count}",
                keep_tool_result=keep_tool_result,
                agent_type="main",
                stream_message_callback=self._intercept_key_message,
            )

            # 清除温度覆盖（无论 LLM 调用成功与否）
            self.llm_client.clear_temperature_override()

            # 监控后检查
            terminate_reason = await self.monitor.post_turn_check(
                response_text=assistant_response_text,
                llm_call_failed=tool_calls == "context_limit" or assistant_response_text is None,
            )

            # 立即检查终止原因
            if terminate_reason:
                task_failed = True
                break

            # 响应循环升级到 INJECT_HINT 时注入强制策略指令 + 温度提升
            if self.monitor.last_loop_action == EscalationAction.INJECT_HINT:
                recent_tools = self._extract_recent_tool_names(message_history)
                hint_text = self.monitor.get_loop_break_hint(recent_tool_names=recent_tools)
                message_history.append(
                    {"role": "user", "content": [{"type": "text", "text": hint_text}]}
                )
                temp_boost = self._monitoring_schema_dict.get("temperature_boost", 0.3)
                temp_cap = self._monitoring_schema_dict.get("temperature_boost_cap", 1.0)
                self.llm_client.set_temperature_boost(boost=temp_boost, cap=temp_cap)

            # Inline Skill: 从 LLM 回复中解析 <next_skills>，下一轮动态注入
            if self.inline_skill_selector and assistant_response_text:
                next_skills = self.inline_skill_selector.update_pending_skills(
                    assistant_response_text
                )
                if next_skills:
                    system_prompt = self.inline_skill_selector.inject_pending_skills(system_prompt)
                    logger.info(f"[InlineSkill] Injected skills for next turn: {next_skills}")

            # 处理 LLM 响应
            if assistant_response_text:
                if should_break:
                    break
            else:
                if tool_calls == "context_limit":
                    # Level 3: 紧急裁剪
                    emergency_count = self.context_manager.apply_emergency(
                        message_history,
                        turn_count,
                        system_prompt,
                        self.llm_client.max_context_length,
                    )
                    if emergency_count > 0:
                        self.task_log.log_step(
                            "context_emergency",
                            f"[CONTEXT L3] Emergency: processed {emergency_count} messages, "
                            f"history now {len(message_history)} messages",
                            "info",
                        )
                        continue  # retry LLM call
                    self.task_log.log_step(
                        "main_agent_context_limit_reached", "Context limit reached", "warning"
                    )
                else:
                    self.task_log.log_step("main_agent", "LLM call failed", "failed")
                task_failed = True
                continue

            # 检查是否有工具调用
            if (
                not tool_calls
                or len(tool_calls) < 2
                or (len(tool_calls[0]) == 0 and len(tool_calls[1]) == 0)
            ):
                logger.debug("LLM did not request tool use, process ends.")
                break

            # 跨轮次去重过滤
            calls_to_execute = tool_calls[0][:max_tool_calls]
            to_execute, cached_results = self.context_manager.filter_duplicate_calls(
                calls_to_execute
            )

            # 执行工具调用（仅非重复的）
            executed_tool_calls = (
                [tool_calls[0][:0], tool_calls[1]] if len(tool_calls) > 1 else [[], []]
            )
            executed_tool_calls[0] = to_execute
            all_tool_results_with_id = []

            if to_execute:
                modified_tool_calls = [to_execute, tool_calls[1] if len(tool_calls) > 1 else []]
                all_tool_results_with_id = await self._execute_tools(
                    modified_tool_calls, max_tool_calls, keep_tool_result
                )

            # 添加缓存结果（dedup 命中的）
            for call_id, cached_content in cached_results:
                all_tool_results_with_id.append((call_id, cached_content))

            # 注册 tool results
            if to_execute and all_tool_results_with_id:
                executed_results = all_tool_results_with_id[: len(to_execute)]
                self.context_manager.register_tool_results(to_execute, executed_results, turn_count)

            # 记录策略摘要
            for call in to_execute:
                self.monitor.record_strategy_summary(
                    f"{call.get('tool_name', '?')}({str(call.get('arguments', ''))[:100]})"
                )

            # 更新消息历史
            tool_calls_exceeded = len(tool_calls[0]) > max_tool_calls
            message_history = self.llm_client.update_message_history(
                message_history, all_tool_results_with_id, tool_calls_exceeded
            )

            # 三级 Context 管理
            action = self.context_manager.manage_context(
                message_history,
                turn_count,
                system_prompt,
                self.llm_client.max_context_length,
            )
            if action == "need_summarize":
                await self.context_manager.apply_summarize(
                    message_history,
                    turn_count,
                    system_prompt,
                    self.llm_client.max_context_length,
                    llm_call_fn=self._context_summarize_call,
                )

            # Hook: on_turn_end
            tool_calls_count = len(tool_calls[0]) if tool_calls and len(tool_calls) > 0 else 0
            hooks.call(
                "on_turn_end",
                HookContext(
                    hook_name="on_turn_end",
                    turn_number=turn_count,
                    tool_calls_count=tool_calls_count,
                    query=task_description,
                    context=self.context,
                ),
            )

            # 反思检查点
            if turn_counter.should_inject_reflection():
                from mem_deep_research_core.core.llm_call_handler import generate_reflection_prompt

                reflection_prompt = generate_reflection_prompt(
                    turn_count, task_description, self.chinese_context
                )
                message_history.append(
                    {"role": "user", "content": [{"type": "text", "text": reflection_prompt}]}
                )
                self.task_log.log_step(
                    "reflection_checkpoint", f"Injected at turn {turn_count}", "info"
                )
                logger.debug(f"[Deep Research] Reflection checkpoint injected at turn {turn_count}")

        # 清理
        self.context_manager.cancel()

        # Hook: on_agent_end
        hooks.call(
            "on_agent_end",
            HookContext(
                hook_name="on_agent_end",
                query=task_description,
                turn_number=turn_counter.current_turn,
                result=task_failed,
                context=self.context,
                extra={
                    "agent_type": "main",
                    "task_failed": task_failed,
                    "turns_used": turn_counter.current_turn,
                },
            ),
        )

        # 退出主 LLM/Agent
        await self.stream_handler.stream_end_llm("main")
        await self.stream_handler.stream_end_agent("main", self.current_agent_id)

        # 记录循环结束
        if turn_counter.is_max_reached():
            if not task_failed:
                task_failed = True
            self.task_log.log_step(
                "max_turns_reached", f"Reached maximum turns ({max_turns})", "warning"
            )
        else:
            self.task_log.log_step(
                "main_loop_completed", f"Completed after {turn_counter.current_turn} turns"
            )

        # 循环终止时清理重复的 assistant 响应
        if task_failed:
            self._deduplicate_trailing_messages(message_history)

        # 注入引用信息到消息历史
        citation_summary = self.context_manager.source_registry.get_citation_summary()
        if citation_summary:
            message_history.append(
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": f"[COLLECTED SOURCES]\n{citation_summary}\n\nPlease include these sources in your final summary where relevant.",
                        }
                    ],
                }
            )
            self.task_log.log_step(
                "citation_injection",
                f"Injected {len(self.context_manager.source_registry.get_all_sources())} sources into message history",
            )

        # 生成最终摘要
        self.task_log.log_step("final_summary", "Generating final summary")

        self.current_agent_id = await self.stream_handler.stream_start_agent("reporter")
        await self.stream_handler.stream_start_llm("reporter")

        final_answer_text = await self._handle_summary(
            system_prompt,
            main_agent_prompt_instance,
            message_history,
            tool_definitions,
            "Final summary generation",
            task_description,
            task_failed,
            agent_type="main",
            task_guidence=task_guidence,
            stream_message_callback=self._streaming_final_message,
            deep_research_cfg=deep_research_cfg,
        )

        return final_answer_text

    async def _execute_tools(
        self,
        tool_calls: list,
        max_tool_calls: int,
        keep_tool_result: int,
    ) -> list:
        """执行工具调用"""
        all_tool_results_with_id = []

        tool_calls_exceeded = len(tool_calls[0]) > max_tool_calls
        if tool_calls_exceeded:
            logger.warning(
                f"[ERROR] Tool call count too high ({len(tool_calls[0])}), "
                f"only processing first {max_tool_calls}"
            )

        for call in tool_calls[0][:max_tool_calls]:
            server_name = call["server_name"]
            tool_name = call["tool_name"]
            arguments = call["arguments"]
            call_id = call["id"]

            if server_name.startswith("agent-"):
                # 子 Agent 调用
                await self.stream_handler.stream_end_llm("main")
                await self.stream_handler.stream_end_agent("main", self.current_agent_id)

                sub_agent_result = await self.sub_agent_runner.run(
                    server_name, str(arguments), keep_tool_result
                )

                tool_result = {
                    "server_name": server_name,
                    "tool_name": tool_name,
                    "result": sub_agent_result,
                }

                self.current_agent_id = await self.stream_handler.stream_start_agent(
                    "main", display_name="Summarizing"
                )
                await self.stream_handler.stream_start_llm("main", display_name="Summarizing")

                tool_result_for_llm = self.output_formatter.format_tool_result_for_user(tool_result)
                all_tool_results_with_id.append((call_id, tool_result_for_llm))
            else:
                # 普通工具调用
                tool_result, _ = await self.tool_executor.execute_single_tool(
                    server_name=server_name,
                    tool_name=tool_name,
                    arguments=arguments,
                    call_id=call_id,
                    agent_name="main",
                )

                tool_result_for_llm = self.output_formatter.format_tool_result_for_user(tool_result)
                all_tool_results_with_id.append((call_id, tool_result_for_llm))

        # 处理失败的工具调用
        if len(tool_calls) > 1 and len(tool_calls[1]) > 0:
            _, failed_results = self.tool_executor.handle_failed_tool_calls(tool_calls[1])
            all_tool_results_with_id.extend(failed_results)

        return all_tool_results_with_id

    async def _context_summarize_call(
        self,
        summarize_system_prompt: str,
        summarize_messages: list,
        purpose: str,
    ) -> str:
        """Level 2 context 压缩的 LLM 调用"""
        response_text, _, _ = await self.llm_handler.handle_llm_call(
            summarize_system_prompt,
            summarize_messages,
            [],
            999,
            purpose,
            agent_type="main",
        )
        return response_text or ""



"""
配置验证 Schema

使用 Pydantic 定义配置结构和验证规则。
"""

from typing import Any, Literal

from pydantic import BaseModel, Field, field_validator


class LLMConfig(BaseModel):
    """LLM 配置验证"""

    provider_class: str = Field(
        ...,
        description="LLM provider 类名",
        examples=["ClaudeOpenRouterClient", "ClaudeAnthropicClient", "GPTOpenAIClient"],
    )
    model_name: str = Field(..., description="模型名称")
    async_client: bool = Field(default=True, description="是否使用异步客户端")

    # Model parameters
    temperature: float = Field(default=0.3, ge=0.0, le=2.0, description="采样温度")
    top_p: float = Field(default=1.0, ge=0.0, le=1.0, description="Top-p 采样")
    top_k: int = Field(default=-1, ge=-1, description="Top-k 采样，-1 表示禁用")
    min_p: float = Field(default=0.0, ge=0.0, le=1.0, description="Min-p 采样")
    max_tokens: int = Field(default=32000, ge=1, description="最大生成 token 数")

    # Timeout and retry
    timeout: int = Field(default=300, ge=1, description="请求超时时间（秒）")
    retry_max_attempts: int = Field(default=3, ge=1, description="最大重试次数")
    retry_strategy: Literal["exponential", "fixed"] = Field(
        default="exponential", description="重试策略"
    )
    retry_multiplier: int = Field(default=2, ge=1, description="重试间隔乘数")
    retry_wait_seconds: int = Field(default=5, ge=1, description="基础重试等待时间")

    # API configuration (optional, can come from env vars)
    openrouter_api_key: str | None = Field(default=None, description="OpenRouter API Key")
    openrouter_base_url: str | None = Field(default=None, description="OpenRouter Base URL")
    openrouter_provider: str | None = Field(default="", description="OpenRouter Provider 偏好")
    anthropic_api_key: str | None = Field(default=None, description="Anthropic API Key")
    api_key: str | None = Field(default=None, description="通用 API Key")
    base_url: str | None = Field(default=None, description="通用 Base URL")

    # Streaming and caching
    enable_streaming: bool = Field(default=True, description="是否启用流式输出")
    disable_cache_control: bool = Field(default=False, description="是否禁用缓存控制")

    # Tool handling
    keep_tool_result: int = Field(
        default=5, description="保留工具结果数，-1 表示全部保留，0+ 保留最近 N 个完整结果"
    )
    oai_tool_thinking: bool = Field(default=False, description="OpenAI 工具思考模式")

    # Context management
    max_context_length: int = Field(
        default=-1, description="模型最大上下文长度(tokens)，-1 表示不限制"
    )

    @field_validator("provider_class")
    @classmethod
    def validate_provider_class(cls, v: str) -> str:
        """验证 provider_class 是有效的 Python 标识符"""
        if not v.isidentifier():
            raise ValueError(f"provider_class must be a valid Python identifier, got: {v}")
        return v

    class Config:
        extra = "allow"


class PromptConfig(BaseModel):
    """新版统一 Prompt 配置"""

    agent_type: Literal["main", "worker"] = Field(default="main", description="Agent 类型")
    tool_format: Literal["xml", "native"] = Field(default="xml", description="工具格式")
    presets: list[str] = Field(default_factory=list, description="预设模块列表")
    templates_dir: str | None = Field(default=None, description="自定义模板目录")
    custom_system_template: str | None = Field(default=None, description="自定义系统提示词模板")
    custom_summarize_template: str | None = Field(default=None, description="自定义总结模板")


class DeepResearchConfig(BaseModel):
    """Deep Research 协议配置"""

    enabled: bool = Field(default=False, description="是否启用深度研究模式")
    reflection_interval: int = Field(default=5, ge=1, description="反思间隔轮次")
    require_explicit_planning: bool = Field(default=True, description="是否需要显式规划")
    auto_planning: bool = Field(default=False, description="是否启用 LLM 自动任务分解")


class InputProcessConfig(BaseModel):
    """输入处理配置"""

    hint_generation: bool = Field(default=False, description="是否启用提示生成")
    hint_llm_base_url: str | None = Field(default=None, description="提示生成 LLM Base URL")


class OutputProcessConfig(BaseModel):
    """输出处理配置"""

    final_answer_extraction: bool = Field(default=False, description="是否提取最终答案")
    final_answer_llm_base_url: str | None = Field(default=None, description="答案提取 LLM Base URL")
    final_answer_model: str | None = Field(default=None, description="答案提取模型")


class SkillSelectionConfig(BaseModel):
    """Skill 选择配置"""

    enabled: bool = Field(default=True, description="是否启用 Skill 选择")

    # 选择方式
    method: Literal["rules", "llm", "inline"] = Field(
        default="rules",
        description=(
            "Skill 选择方式: "
            "rules=仅规则匹配, "
            "llm=额外 LLM 调用选择, "
            "inline=LLM 在回复中声明下一轮 skill（零额外开销）"
        ),
    )

    # LLM 选择配置（method=llm 时使用）
    model: str = Field(default="gpt-4o-mini", description="用于 Skill 选择的模型")
    fallback_to_rules: bool = Field(default=True, description="LLM 失败时是否降级到规则匹配")

    # 通用配置
    max_skills: int = Field(default=3, ge=0, le=10, description="最大选择 Skill 数量")


class MonitoringConfigSchema(BaseModel):
    """执行监控配置"""

    # 停滞检测
    stall_detection_threshold: float = Field(
        default=120.0, ge=0.0, description="停滞检测阈值（秒）"
    )
    stall_terminate_multiplier: float = Field(default=2.0, ge=1.0, description="停滞终止倍数")
    max_total_time: float = Field(default=600.0, ge=0.0, description="最大总运行时间（秒）")

    # 空响应检测
    max_consecutive_empty_turns: int = Field(default=3, ge=1, description="连续空响应终止阈值")

    # 响应循环检测
    enable_loop_detection: bool = Field(default=True, description="是否启用重复响应检测")
    loop_detection_text_length: int = Field(default=500, ge=100, description="循环检测文本截取长度")
    loop_escalation_terminate_threshold: int = Field(
        default=3, ge=1, description="响应循环终止阈值"
    )
    response_hash_window_size: int = Field(default=8, ge=2, description="滑动窗口大小")
    response_hash_repeat_threshold: int = Field(default=3, ge=2, description="窗口内 hash 重复阈值")

    # 工具循环
    max_tool_loop_retries: int = Field(default=2, ge=0, description="工具循环最大重试次数")

    # 温度提升（响应循环升级时使用）
    temperature_boost: float = Field(
        default=0.3, ge=0.0, le=1.0, description="循环检测时温度提升量"
    )
    temperature_boost_cap: float = Field(default=1.0, ge=0.0, le=2.0, description="温度提升上限")

    # 工具结果限制
    scrape_max_length: int = Field(default=20000, ge=1000, description="Scrape 结果最大长度")


class ContextManagerConfig(BaseModel):
    """Context Manager 配置

    三级 Context 管理策略:
      Level 1 (Compact): token-aware 摘要替换，零 LLM 成本
      Level 2 (Summarize): LLM 压缩旧历史为一条摘要
      Level 3 (Emergency): 二分删除中间消息
    """

    enable_dedup: bool = Field(default=True, description="是否启用跨轮次 tool call 去重")
    enable_compact: bool = Field(default=True, description="是否启用 Level 1 摘要替换")

    # Level 1: 摘要替换
    compact_at_ratio: float = Field(
        default=0.6, ge=0.0, le=1.0, description="token 占比超过此值时触发 compact"
    )
    compact_keep_recent: int = Field(default=3, ge=1, description="至少保留最近 N 轮完整结果")

    # Level 2: LLM 压缩
    summarize_at_ratio: float = Field(
        default=0.8, ge=0.0, le=1.0, description="token 占比超过此值时触发 LLM 压缩"
    )

    # Token 估算
    chars_per_token: float = Field(
        default=3.5, gt=0.0, description="无 tiktoken 时的 fallback 估算比例"
    )

    # 兼容旧配置
    mask_after_n_turns: int = Field(
        default=5, ge=1, description="[已废弃] 旧配置，被 compact_keep_recent 取代"
    )
    enable_masking: bool = Field(
        default=True, description="[已废弃] 旧配置，被 enable_compact 取代"
    )


class InterceptorConfig(BaseModel):
    """消息拦截器配置"""

    preset: str = Field(default="default", description="拦截器预设名称")

    class Config:
        extra = "allow"


class MainAgentConfig(BaseModel):
    """主 Agent 配置"""

    # 新版 prompt 配置
    prompt: PromptConfig = Field(default_factory=PromptConfig, description="Prompt 配置")

    llm: LLMConfig = Field(..., description="LLM 配置")
    tool_config: list[str] = Field(default_factory=list, description="工具配置列表")
    tool_blacklist: list = Field(default_factory=list, description="工具黑名单")

    # Execution limits
    max_turns: int = Field(default=20, ge=1, description="最大对话轮次")
    max_tool_calls_per_turn: int = Field(default=10, ge=1, description="每轮最大工具调用数")
    keep_tool_result: int = Field(default=-1, description="保留工具结果数")

    # Deep Research
    deep_research: DeepResearchConfig = Field(
        default_factory=DeepResearchConfig, description="Deep Research 配置"
    )

    # Processing
    input_process: InputProcessConfig = Field(
        default_factory=InputProcessConfig, description="输入处理配置"
    )
    output_process: OutputProcessConfig = Field(
        default_factory=OutputProcessConfig, description="输出处理配置"
    )

    # Interceptor
    interceptor: InterceptorConfig = Field(
        default_factory=InterceptorConfig, description="消息拦截器配置"
    )

    # Skill Selection
    skill_selection: SkillSelectionConfig = Field(
        default_factory=SkillSelectionConfig, description="Skill LLM 选择配置"
    )

    # Context Manager
    context_manager: ContextManagerConfig = Field(
        default_factory=ContextManagerConfig, description="Context Manager 配置"
    )

    # Monitoring
    monitoring: MonitoringConfigSchema = Field(
        default_factory=MonitoringConfigSchema, description="执行监控配置"
    )

    # Context
    add_message_id: bool = Field(default=True, description="是否添加消息 ID")
    chinese_context: bool = Field(default=False, description="是否使用中文上下文")

    class Config:
        extra = "allow"


class BenchmarkConfig(BaseModel):
    """Benchmark 配置"""

    name: str = Field(default="custom", description="Benchmark 名称")


class AgentConfig(BaseModel):
    """完整的 Agent 配置"""

    main_agent: MainAgentConfig = Field(..., description="主 Agent 配置")
    sub_agents: dict[str, Any] | None = Field(default=None, description="子 Agent 配置")
    benchmark: BenchmarkConfig = Field(
        default_factory=BenchmarkConfig, description="Benchmark 配置"
    )
    output_dir: str = Field(default="logs/", description="输出目录")

    class Config:
        extra = "allow"  # 允许额外字段（如 defaults, env 等 Hydra 特定字段）


class ToolConfig(BaseModel):
    """工具配置"""

    name: str = Field(..., description="工具名称")
    tool_command: str = Field(..., description="工具启动命令")
    args: list[str] = Field(default_factory=list, description="命令参数")
    env: dict[str, str] = Field(default_factory=dict, description="环境变量")

    @field_validator("tool_command")
    @classmethod
    def validate_tool_command(cls, v: str) -> str:
        """验证工具命令不为空"""
        if not v.strip():
            raise ValueError("tool_command cannot be empty")
        return v


def validate_agent_config(config_dict: dict[str, Any]) -> AgentConfig:
    """
    验证 Agent 配置字典。

    Args:
        config_dict: 配置字典（通常从 YAML 加载）

    Returns:
        验证后的 AgentConfig 对象

    Raises:
        pydantic.ValidationError: 配置验证失败
    """
    return AgentConfig.model_validate(config_dict)


def validate_tool_config(config_dict: dict[str, Any]) -> ToolConfig:
    """
    验证工具配置字典。

    Args:
        config_dict: 配置字典

    Returns:
        验证后的 ToolConfig 对象

    Raises:
        pydantic.ValidationError: 配置验证失败
    """
    return ToolConfig.model_validate(config_dict)



"""
子 Agent 运行模块

处理子 Agent 的完整生命周期，包括初始化、执行循环和结果汇总。
"""

import logging
import sys
import uuid
from collections.abc import Callable
from typing import Any

from omegaconf import DictConfig

from mem_deep_research_core.core.hooks import HookContext, hooks
from mem_deep_research_core.core.tool_executor import ToolExecutor
from mem_deep_research_core.core.tool_result_formatter import ToolResultFormatter
from mem_deep_research_core.llm.provider_client_base import LLMProviderClientBase
from mem_deep_research_core.mem_deep_research_logging.logger import truncate_for_log
from mem_deep_research_core.mem_deep_research_logging.task_tracer import TaskTracer
from mem_deep_research_core.tool.manager import ToolManager
from mem_deep_research_core.utils.external_loader import external_loader
from mem_deep_research_core.utils.io_utils import OutputFormatter
from mem_deep_research_core.utils.tool_utils import _load_agent_prompt

logger = logging.getLogger("mem_deep_research")


class SubAgentRunner:
    """子 Agent 运行器"""

    def __init__(
        self,
        sub_agent_tool_managers: dict[str, ToolManager],
        sub_agent_llm_client: LLMProviderClientBase,
        output_formatter: OutputFormatter,
        cfg: DictConfig,
        task_log: TaskTracer,
        context: dict[str, Any] | None = None,
        chinese_context: bool = False,
        # 流式回调
        stream_start_agent: Callable | None = None,
        stream_end_agent: Callable | None = None,
        stream_start_llm: Callable | None = None,
        stream_end_llm: Callable | None = None,
        stream_tool_call: Callable | None = None,
        stream_tool_reasoning: Callable | None = None,
        stream_usage_info: Callable | None = None,
        # LLM 调用回调
        handle_llm_call: Callable | None = None,
        handle_summary: Callable | None = None,
        # 消息拦截回调
        intercept_key_message: Callable | None = None,
    ):
        """
        初始化子 Agent 运行器

        Args:
            sub_agent_tool_managers: 子 Agent 工具管理器字典
            sub_agent_llm_client: 子 Agent 的 LLM 客户端
            output_formatter: 输出格式化器
            cfg: 配置
            task_log: 任务日志
            context: 用户上下文
            chinese_context: 是否使用中文上下文
            stream_*: 各种流式输出回调
            handle_llm_call: LLM 调用处理回调
            handle_summary: 摘要生成回调
            intercept_key_message: 消息拦截回调
        """
        self.sub_agent_tool_managers = sub_agent_tool_managers
        self.sub_agent_llm_client = sub_agent_llm_client
        self.output_formatter = output_formatter
        self.cfg = cfg
        self.task_log = task_log
        self.context = context or {}
        self.chinese_context = chinese_context

        # 流式回调
        self.stream_start_agent = stream_start_agent
        self.stream_end_agent = stream_end_agent
        self.stream_start_llm = stream_start_llm
        self.stream_end_llm = stream_end_llm
        self.stream_tool_call = stream_tool_call
        self.stream_tool_reasoning = stream_tool_reasoning
        self.stream_usage_info = stream_usage_info

        # LLM 调用回调
        self.handle_llm_call = handle_llm_call
        self.handle_summary = handle_summary

        # 消息拦截回调
        self.intercept_key_message = intercept_key_message

        # 工具结果格式化器
        self.tool_result_formatter = ToolResultFormatter(context)

        # 预缓存的工具定义
        self._cached_tool_definitions: dict[str, list[dict]] | None = None

    def set_cached_tool_definitions(self, definitions: dict[str, list[dict]]):
        """设置预缓存的子 Agent 工具定义"""
        self._cached_tool_definitions = definitions

    async def _get_tool_definitions(self, sub_agent_name: str) -> list[dict]:
        """获取子 Agent 的工具定义"""
        if self._cached_tool_definitions:
            return self._cached_tool_definitions.get(sub_agent_name, [])

        # 动态获取
        tool_manager = self.sub_agent_tool_managers.get(sub_agent_name)
        if tool_manager:
            return await tool_manager.get_all_tool_definitions()
        return []

    def _create_tool_executor(self, sub_agent_name: str) -> ToolExecutor:
        """为子 Agent 创建工具执行器"""
        tool_manager = self.sub_agent_tool_managers.get(sub_agent_name)
        if not tool_manager:
            raise ValueError(f"Tool manager not found for sub-agent: {sub_agent_name}")

        return ToolExecutor(
            tool_manager=tool_manager,
            output_formatter=self.output_formatter,
            tool_result_formatter=self.tool_result_formatter,
            context=self.context,
            stream_tool_call=self.stream_tool_call,
            stream_tool_reasoning=self.stream_tool_reasoning,
            stream_usage_info=self.stream_usage_info,
        )

    def _load_sub_agent_prompt(self, sub_agent_name: str):
        """加载子 Agent 的提示词实例"""
        if not self.cfg.sub_agents or sub_agent_name not in self.cfg.sub_agents:
            raise ValueError(f"Sub-agent {sub_agent_name} not found in configuration")

        sub_agent_cfg = self.cfg.sub_agents[sub_agent_name]

        # 获取 prompt 配置
        prompt_cfg = {}
        if hasattr(sub_agent_cfg, "prompt") and sub_agent_cfg.prompt:
            prompt_cfg = dict(sub_agent_cfg.prompt)
        # 子 Agent 默认为 worker 类型
        if "agent_type" not in prompt_cfg:
            prompt_cfg["agent_type"] = "worker"

        return _load_agent_prompt(prompt_cfg)

    def _generate_system_prompt(
        self,
        sub_agent_name: str,
        tool_definitions: list[dict],
        task_description: str,
    ) -> str:
        """生成子 Agent 的系统提示词"""
        prompt_instance = self._load_sub_agent_prompt(sub_agent_name)

        system_prompt = prompt_instance.generate_system_prompt_with_mcp_tools(
            mcp_servers=tool_definitions,
            chinese_context=self.chinese_context,
        )

        # 注入 Skills
        skill_injector = external_loader.get_skill_injector()
        if skill_injector and task_description:
            tools_to_use = [t.get("name", "") for t in tool_definitions if isinstance(t, dict)]
            system_prompt = skill_injector.inject_skills(
                base_prompt=system_prompt,
                query=task_description,
                context=self.context,
                tools_to_use=tools_to_use,
            )

        return system_prompt

    async def run(
        self,
        sub_agent_name: str,
        task_description: str,
        keep_tool_result: int = -1,
    ) -> str:
        """
        运行子 Agent

        Args:
            sub_agent_name: 子 Agent 名称
            task_description: 任务描述
            keep_tool_result: 保留工具结果数量

        Returns:
            str: 子 Agent 的最终回答
        """
        logger.debug(f"\n=== Starting Sub Agent {sub_agent_name} ===")
        task_description += "\n\nPlease provide the answer and detailed supporting information of the subtask given to you."
        logger.debug(f"Subtask: {task_description}")

        # 发送子 Agent 开始事件
        display_name = sub_agent_name.replace("agent-", "")
        sub_agent_id = None
        if self.stream_start_agent:
            sub_agent_id = await self.stream_start_agent(display_name)
        if self.stream_start_llm:
            await self.stream_start_llm(display_name)

        # 开始新的子 Agent 会话
        self.task_log.start_sub_agent_session(sub_agent_name, task_description)

        # 初始化消息历史
        initial_user_content = [{"type": "text", "text": task_description}]
        message_history = [{"role": "user", "content": initial_user_content}]

        # 获取工具定义
        tool_definitions = await self._get_tool_definitions(sub_agent_name)
        self.task_log.log_step(f"get_sub_{sub_agent_name}_tool_definitions", f"{tool_definitions}")

        if not tool_definitions:
            logger.debug(
                "Warning: Failed to get any tool definitions. LLM may not be able to use tools."
            )
            self.task_log.log_step(
                f"{sub_agent_name}_no_tools",
                f"No tool definitions available for {sub_agent_name}",
                "warning",
            )

        # 生成系统提示词
        prompt_instance = self._load_sub_agent_prompt(sub_agent_name)
        system_prompt = self._generate_system_prompt(
            sub_agent_name, tool_definitions, task_description
        )

        # 创建工具执行器
        tool_executor = self._create_tool_executor(sub_agent_name)

        # 获取配置
        sub_agent_cfg = self.cfg.sub_agents[sub_agent_name]
        max_turns = sub_agent_cfg.max_turns
        if max_turns < 0:
            max_turns = sys.maxsize
        max_tool_calls = sub_agent_cfg.max_tool_calls_per_turn

        # Hook: on_agent_start
        hooks.call(
            "on_agent_start",
            HookContext(
                hook_name="on_agent_start",
                query=task_description,
                context=self.context,
                extra={"agent_type": sub_agent_name},
            ),
        )

        # 执行循环
        turn_count = 0
        task_failed = False

        while turn_count < max_turns:
            turn_count += 1
            logger.debug(f"\n--- Sub Agent {sub_agent_name} Turn {turn_count} ---")

            # Hook: on_turn_start
            hooks.call(
                "on_turn_start",
                HookContext(
                    hook_name="on_turn_start",
                    turn_number=turn_count,
                    query=task_description,
                    context=self.context,
                    extra={"agent_type": sub_agent_name},
                ),
            )
            self.task_log.save()

            # 调用 LLM
            if not self.handle_llm_call:
                logger.error("handle_llm_call callback not set")
                break

            assistant_response_text, should_break, tool_calls = await self.handle_llm_call(
                system_prompt,
                message_history,
                tool_definitions,
                turn_count,
                f"Sub agent {sub_agent_name} turn {turn_count}",
                keep_tool_result=keep_tool_result,
                agent_type=sub_agent_name,
                stream_message_callback=self.intercept_key_message,
            )

            # 处理 LLM 响应
            if assistant_response_text:
                if should_break:
                    self.task_log.log_step(
                        "sub_agent_early_termination",
                        f"Sub agent {sub_agent_name} terminated early on turn {turn_count}",
                    )
                    break
            else:
                if tool_calls == "context_limit":
                    self.task_log.log_step(
                        "sub_agent_context_limit_reached",
                        f"Sub agent {sub_agent_name} context limit reached, jumping to summary",
                        "warning",
                    )
                else:
                    self.task_log.log_step(
                        "sub_agent_llm_call_failed",
                        "LLM call failed",
                        "failed",
                    )
                task_failed = True
                break

            # 检查是否有工具调用
            if (
                tool_calls is None
                or len(tool_calls) < 2
                or (len(tool_calls[0]) == 0 and len(tool_calls[1]) == 0)
            ):
                logger.debug(f"Sub Agent {sub_agent_name} did not request tool use, ending task.")
                self.task_log.log_step(
                    "sub_agent_no_tool_calls",
                    f"No tool calls found in sub agent {sub_agent_name}, ending on turn {turn_count}",
                )
                break

            # 执行工具调用
            (
                tool_calls_data,
                tool_results_with_id,
                exceeded,
            ) = await tool_executor.execute_tool_calls(
                tool_calls[0],
                max_tool_calls,
                agent_name=sub_agent_name,
            )

            # 处理失败的工具调用
            if len(tool_calls) > 1 and len(tool_calls[1]) > 0:
                failed_data, failed_results = tool_executor.handle_failed_tool_calls(tool_calls[1])
                tool_calls_data.extend(failed_data)
                tool_results_with_id.extend(failed_results)

            # 更新消息历史
            message_history = self.sub_agent_llm_client.update_message_history(
                message_history, tool_results_with_id, exceeded
            )

            # Hook: on_turn_end
            tool_calls_count = len(tool_calls[0]) if tool_calls and len(tool_calls) > 0 else 0
            hooks.call(
                "on_turn_end",
                HookContext(
                    hook_name="on_turn_end",
                    turn_number=turn_count,
                    tool_calls_count=tool_calls_count,
                    query=task_description,
                    context=self.context,
                    extra={"agent_type": sub_agent_name},
                ),
            )

        # Hook: on_agent_end
        hooks.call(
            "on_agent_end",
            HookContext(
                hook_name="on_agent_end",
                query=task_description,
                turn_number=turn_count,
                result=task_failed,
                context=self.context,
                extra={
                    "agent_type": sub_agent_name,
                    "task_failed": task_failed,
                    "turns_used": turn_count,
                },
            ),
        )

        # 记录循环结束
        logger.debug(f"\n=== Sub Agent {sub_agent_name} Completed ({turn_count} turns) ===")

        if turn_count >= max_turns:
            if not task_failed:
                task_failed = True
            self.task_log.log_step(
                "sub_agent_max_turns_reached",
                f"Sub agent {sub_agent_name} reached maximum turns ({max_turns})",
                "warning",
            )
        else:
            self.task_log.log_step(
                "sub_agent_loop_completed",
                f"Sub agent {sub_agent_name} loop completed after {turn_count} turns",
            )

        # 生成最终摘要
        self.task_log.log_step(
            "sub_agent_final_summary",
            f"Generating sub agent {sub_agent_name} final summary",
        )

        if self.stream_tool_call:
            await self.stream_tool_call("Partial Summary", {}, tool_call_id=str(uuid.uuid4()))

        # 使用 context limit 重试逻辑生成最终摘要
        final_answer_text = ""
        if self.handle_summary:
            final_answer_text = await self.handle_summary(
                system_prompt,
                prompt_instance,
                message_history,
                tool_definitions,
                f"Sub agent {sub_agent_name} final summary",
                task_description,
                task_failed,
                agent_type=sub_agent_name,
                stream_message_callback=self.intercept_key_message,
            )

        if final_answer_text:
            self.task_log.log_step(
                "sub_agent_final_answer",
                f"Sub agent {sub_agent_name} final answer generated successfully",
            )
        else:
            final_answer_text = f"No final answer generated by sub agent {sub_agent_name}."
            self.task_log.log_step(
                "sub_agent_final_answer",
                f"Failed to generate sub agent {sub_agent_name} final answer",
                "failed",
            )

        logger.debug(
            f"Sub Agent {sub_agent_name} Final Answer: {truncate_for_log(final_answer_text)}"
        )

        # 保存消息历史
        self.task_log.sub_agent_message_history_sessions[
            self.task_log.current_sub_agent_session_id
        ] = {"system_prompt": system_prompt, "message_history": message_history}
        self.task_log.save()

        # 结束子 Agent 会话
        self.task_log.end_sub_agent_session(sub_agent_name)
        self.task_log.log_step(
            "sub_agent_completed", f"Sub agent {sub_agent_name} completed", "info"
        )

        # 发送子 Agent 结束事件
        if self.stream_end_llm:
            await self.stream_end_llm(display_name)
        if self.stream_end_agent and sub_agent_id:
            await self.stream_end_agent(display_name, sub_agent_id)

        return final_answer_text



import asyncio
import dataclasses
import os
from collections.abc import Callable

from anthropic import (
    NOT_GIVEN,
    Anthropic,
    AsyncAnthropic,
)
from omegaconf import DictConfig

# tenacity imports removed - using configurable retry from base class
from mem_deep_research_core.llm.provider_client_base import LLMProviderClientBase
from mem_deep_research_core.llm.util import collect_anthropic_stream
from mem_deep_research_core.mem_deep_research_logging.logger import (
    bootstrap_logger,
    truncate_for_log,
)

LOGGER_LEVEL = os.getenv("LOGGER_LEVEL", "INFO")
logger = bootstrap_logger(level=LOGGER_LEVEL)


@dataclasses.dataclass
class ClaudeAnthropicClient(LLMProviderClientBase):
    def __post_init__(self):
        super().__post_init__()

    def _create_client(self, config: DictConfig):
        """Create Anthropic client"""
        api_key = self.cfg.llm.anthropic_api_key

        if self.async_client:
            return AsyncAnthropic(
                api_key=api_key,
                base_url=self.cfg.llm.anthropic_base_url,
                timeout=self.timeout,
            )
        else:
            return Anthropic(
                api_key=api_key,
                base_url=self.cfg.llm.anthropic_base_url,
                timeout=self.timeout,
            )

    async def _create_message(
        self,
        system_prompt,
        messages,
        tools_definitions,
        keep_tool_result: int = -1,
        stream_message_callback: Callable | None = None,
    ):
        """
        Send message to Anthropic API.
        :param system_prompt: System prompt string.
        :param messages: Message history list.
        :return: Anthropic API response object or None (if error).
        """
        logger.debug(f" Calling LLM ({'async' if self.async_client else 'sync'})")

        messages_copy = self._remove_tool_result_from_messages(messages, keep_tool_result)

        processed_messages = self._apply_cache_control(messages_copy)

        try:
            # Apply configurable retry decorator to the API call
            retry_decorator = self.get_retry_decorator()
            create_completion_with_retry = retry_decorator(self._create_completion)
            response = await create_completion_with_retry(system_prompt, processed_messages)

            # If streaming is enabled, collect all chunks into a complete response
            if self.enable_streaming:
                response = await collect_anthropic_stream(response)

            logger.debug(f"LLM call status: {getattr(response, 'stop_reason', 'N/A')}")
            return response
        except asyncio.CancelledError:
            logger.exception("[WARNING] LLM API call was cancelled during execution")
            raise  # Re-raise to allow decorator to log it
        except Exception as e:
            logger.exception("Anthropic LLM endpoint failed")
            raise e

    async def _create_completion(self, system_prompt, processed_messages):
        """Helper to create a completion, handling async and sync calls."""
        if self.async_client:
            return await self.client.messages.create(
                model=self.model_name,
                temperature=self.get_effective_temperature(),
                top_p=self.top_p if self.top_p != 1.0 else NOT_GIVEN,
                top_k=self.top_k if self.top_k != -1 else NOT_GIVEN,
                max_tokens=self.max_tokens,
                system=[
                    {
                        "type": "text",
                        "text": system_prompt,
                        "cache_control": {"type": "ephemeral"},
                    }
                ],
                messages=processed_messages,
                stream=self.enable_streaming,
            )
        else:
            return self.client.messages.create(
                model=self.model_name,
                temperature=self.get_effective_temperature(),
                top_p=self.top_p if self.top_p != 1.0 else NOT_GIVEN,
                top_k=self.top_k if self.top_k != -1 else NOT_GIVEN,
                max_tokens=self.max_tokens,
                system=[
                    {
                        "type": "text",
                        "text": system_prompt,
                        "cache_control": {"type": "ephemeral"},
                    }
                ],
                messages=processed_messages,
                stream=self.enable_streaming,
            )

    def process_llm_response(
        self, llm_response, message_history, agent_type="main"
    ) -> tuple[str, bool]:
        """Process Anthropic LLM response"""
        if not llm_response:
            logger.debug("[ERROR] LLM call failed, skipping this response.")
            return "", True

        if not hasattr(llm_response, "content") or not llm_response.content:
            logger.debug("[ERROR] LLM response is empty or doesn't contain content.")
            return "", True

        # Extract response content
        assistant_response_text = ""
        assistant_response_content = []

        for block in llm_response.content:
            if block.type == "text":
                assistant_response_text += block.text + "\n"
                assistant_response_content.append({"type": "text", "text": block.text})
            elif block.type == "tool_use":
                assistant_response_content.append(
                    {
                        "type": "tool_use",
                        "id": block.id,
                        "name": block.name,
                        "input": block.input,
                    }
                )

        message_history.append({"role": "assistant", "content": assistant_response_content})

        logger.debug(f"LLM Response: {truncate_for_log(assistant_response_text)}")

        return assistant_response_text, False

    def extract_tool_calls_info(self, llm_response, assistant_response_text):
        """Extract tool call information from Anthropic LLM response"""
        from mem_deep_research_core.utils.parsing_utils import parse_llm_response_for_tool_calls

        # For Anthropic, parse tool calls from the response text
        return parse_llm_response_for_tool_calls(assistant_response_text)

    def update_message_history(
        self, message_history, tool_call_info, tool_calls_exceeded: bool = False
    ):
        """Update message history with tool calls data (llm client specific)"""

        merged_text = "\n".join(
            [item[1]["text"] for item in tool_call_info if item[1]["type"] == "text"]
        )

        message_history.append(
            {
                "role": "user",
                "content": [{"type": "text", "text": merged_text}],
            }
        )

        return message_history

    def handle_max_turns_reached_summary_prompt(self, message_history, summary_prompt):
        """Handle max turns reached summary prompt"""
        # if message_history[-1]["role"] == "user":
        #     last_user_message = message_history.pop()
        #     return (
        #         last_user_message["content"][0]["text"]
        #         + "\n*************\n"
        #         + summary_prompt
        #     )
        # else:
        return summary_prompt

    def _apply_cache_control(self, messages):
        """Apply cache control to the last user message and system message (if applicable)"""
        cached_messages = []
        user_turns_processed = 0
        for turn in reversed(messages):
            if turn["role"] == "user" and user_turns_processed < 1:
                # Add ephemeral cache control to the text part of the last user message
                new_content = []
                processed_text = False
                # Check if content is a list
                if isinstance(turn.get("content"), list):
                    # see example here
                    # https://docs.anthropic.com/en/docs/build-with-claude/prompt-caching
                    for item in turn["content"]:
                        if (
                            item.get("type") == "text"
                            and len(item.get("text")) > 0
                            and not processed_text
                        ):
                            # Copy and add cache control
                            text_item = item.copy()
                            text_item["cache_control"] = {"type": "ephemeral"}
                            new_content.append(text_item)
                            processed_text = True
                        else:
                            # Other types of content (like image) copied directly
                            new_content.append(item.copy())
                    cached_messages.append({"role": "user", "content": new_content})
                else:
                    # If content is not a list (e.g., plain text), add as is without cache control
                    # Or adjust logic as needed
                    logger.debug(
                        "Warning: User message content is not in expected list format, cache control not applied."
                    )
                    cached_messages.append(turn)

                user_turns_processed += 1
            else:
                # Add other messages directly
                cached_messages.append(turn)
        return list(reversed(cached_messages))



import asyncio
import json
import logging
import os
import socket
import threading
import uuid
from contextlib import contextmanager
from contextvars import ContextVar
from functools import lru_cache
from pathlib import Path
from typing import Literal

import hydra
import zmq
import zmq.asyncio
from rich.console import Console
from rich.logging import RichHandler

# Task context variable
TASK_CONTEXT_VAR: ContextVar[str | None] = ContextVar("CURRENT_TASK_ID", default=None)

# Maximum length for debug log content (to prevent log bloat)
DEBUG_LOG_MAX_LENGTH = 500


def truncate_for_log(content: str, max_length: int = DEBUG_LOG_MAX_LENGTH) -> str:
    """Truncate long content for logging to prevent log bloat.

    Args:
        content: The content to potentially truncate
        max_length: Maximum length before truncation (default: 500)

    Returns:
        Truncated content with length indicator if truncated
    """
    if not content:
        return content
    content_str = str(content)
    if len(content_str) <= max_length:
        return content_str
    return f"{content_str[:max_length]}... ({len(content_str)} chars total)"


# Context variables for logging identifiers
trace_id_var: ContextVar[str] = ContextVar("trace_id", default="")
chat_id_var: ContextVar[str] = ContextVar("chat_id", default="")
user_id_var: ContextVar[str] = ContextVar("user_id", default="")
message_id_var: ContextVar[str] = ContextVar("message_id", default="")

# Global variable to store the actual ZMQ address being used
_ZMQ_ADDRESS: str = "tcp://127.0.0.1:6000"


def generate_trace_id() -> str:
    """Generate a new trace ID"""
    return str(uuid.uuid4())


def get_trace_id() -> str:
    """Get current trace ID from context"""
    return trace_id_var.get()


def set_trace_id(trace_id: str = None) -> str:
    """Set trace ID in context, generate new one if not provided"""
    if trace_id is None:
        trace_id = generate_trace_id()
    trace_id_var.set(trace_id)
    return trace_id


def get_chat_id() -> str:
    """Get current chat ID from context"""
    return chat_id_var.get()


def set_chat_id(chat_id: str) -> str:
    """Set chat ID in context"""
    chat_id_var.set(chat_id)
    return chat_id


def get_user_id() -> str:
    """Get current user ID from context"""
    return user_id_var.get()


def set_user_id(user_id: str) -> str:
    """Set user ID in context"""
    user_id_var.set(user_id)
    return user_id


def get_message_id() -> str:
    """Get current message ID from context"""
    return message_id_var.get()


def set_message_id(message_id: str) -> str:
    """Set message ID in context"""
    message_id_var.set(message_id)
    return message_id


def find_available_port(start_port: int = 6000, max_attempts: int = 10) -> int:
    """Find an available port starting from start_port."""
    for port in range(start_port, start_port + max_attempts):
        try:
            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                s.bind(("127.0.0.1", port))
                return port
        except OSError:
            continue
    raise RuntimeError(
        f"Could not find an available port in range {start_port}-{start_port + max_attempts - 1}"
    )


def get_zmq_address() -> str:
    """Get the current ZMQ address."""
    return _ZMQ_ADDRESS


def set_zmq_address(address: str) -> None:
    """Set the ZMQ address."""
    global _ZMQ_ADDRESS
    _ZMQ_ADDRESS = address


def _extract_port_from_address(addr: str) -> int:
    """Extract port number from ZMQ address."""
    try:
        return int(addr.split(":")[-1])
    except (ValueError, IndexError):
        return 6000


def _bind_zmq_socket(sock, bind_addr: str) -> str:
    """Bind ZMQ socket to an available port and return the actual address."""
    port = _extract_port_from_address(bind_addr)

    try:
        available_port = find_available_port(port)
        actual_addr = f"tcp://127.0.0.1:{available_port}"
        sock.bind(actual_addr)
        return actual_addr
    except RuntimeError:
        # Fallback to random port
        port = sock.bind_to_random_port("tcp://127.0.0.1")
        return f"tcp://127.0.0.1:{port}"


class ZMQLogHandler(logging.Handler):
    def __init__(self, addr=None, tool_name="unknown_tool"):
        super().__init__()
        ctx = zmq.Context()
        self.sock = ctx.socket(zmq.PUSH)

        # Use the global ZMQ address if no specific address is provided
        if addr is None:
            addr = get_zmq_address()

        # Try to connect to the address
        try:
            self.sock.connect(addr)
            logging.getLogger(__name__).info(f"ZMQ handler connected to: {addr}")
        except zmq.error.ZMQError as e:
            # If connection fails, disable the handler
            logging.getLogger(__name__).warning(f"Could not connect to ZMQ listener at {addr}: {e}")
            logging.getLogger(__name__).warning("Disabling ZMQ logging for this handler")
            self.sock = None

        self.task_id = os.environ.get("TASK_ID", "0")
        self.tool_name = tool_name

    def emit(self, record):
        if self.sock is None:
            return

        try:
            msg = f"{record.getMessage()}"
            self.sock.send_string(f"{self.task_id}||{self.tool_name}||{msg}")
        except Exception:
            self.handleError(record)


async def zmq_log_listener(bind_addr="tcp://127.0.0.1:6000"):
    ctx = zmq.asyncio.Context()
    sock = ctx.socket(zmq.PULL)

    # Bind to available port
    actual_addr = _bind_zmq_socket(sock, bind_addr)
    set_zmq_address(actual_addr)
    logging.getLogger(__name__).info(f"ZMQ listener bound to: {actual_addr}")

    root_logger = logging.getLogger()

    while True:
        raw = await sock.recv_string()
        if "||" in raw:
            task_id, tool_name, msg = raw.split("||", 2)

            record = root_logger.makeRecord(
                name=f"[TOOL] {tool_name}",
                level=logging.INFO,
                fn="",
                lno=0,
                msg=msg,
                args=(),
                exc_info=None,
            )
            record.task_id = task_id

            root_logger.handle(record)
        else:
            root_logger.info(raw)


def start_zmq_listener():
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(zmq_log_listener())


def setup_mcp_logging(level="INFO", addr=None, tool_name="unknown_tool"):
    root = logging.getLogger()
    root.setLevel(level)

    # Remove root handlers
    for h in root.handlers[:]:
        root.removeHandler(h)
        h.close()

    # Remove all handlers from fastmcp child loggers
    for _name, logger in logging.Logger.manager.loggerDict.items():
        if isinstance(logger, logging.Logger):
            for h in logger.handlers[:]:
                logger.removeHandler(h)
                h.close()
            logger.propagate = True  # Ensure bubbling to root

    # Re-add the ZMQ handler (will use global address if addr is None)
    handler = ZMQLogHandler(addr=addr, tool_name=tool_name)
    handler.setFormatter(logging.Formatter("[TOOL] %(asctime)s %(levelname)s: %(message)s"))
    root.addHandler(handler)


def setup_log_record_factory():
    old_factory = logging.getLogRecordFactory()

    def record_factory(*args, **kwargs):
        record = old_factory(*args, **kwargs)
        record.task_id = TASK_CONTEXT_VAR.get()
        return record

    logging.setLogRecordFactory(record_factory)


class TaskFilter(logging.Filter):
    def __init__(self, task_id: str):
        super().__init__()
        self.task_id = task_id

    def filter(self, record: logging.LogRecord) -> bool:
        return getattr(record, "task_id", None) == self.task_id


def make_task_logger(task_id: str, log_dir: Path) -> logging.Handler:
    log_dir.mkdir(parents=True, exist_ok=True)
    file_path = log_dir / f"task_{task_id}.log"
    fh = logging.FileHandler(file_path, encoding="utf-8")
    fmt = logging.Formatter("%(asctime)s [%(levelname)s] %(name)s: %(message)s")
    fh.setFormatter(fmt)
    fh.addFilter(TaskFilter(task_id))
    logging.getLogger().addHandler(fh)
    return fh


class JSONFormatter(logging.Formatter):
    """Custom JSON formatter that includes trace_id, chat_id, user_id, and message_id"""

    def format(self, record):
        log_obj = {
            "timestamp": self.formatTime(record, datefmt="%Y-%m-%d %H:%M:%S"),
            "level": record.levelname,
            "logger": record.name,
            "file": record.filename,
            "line": record.lineno,
            "message": record.getMessage(),
            "trace_id": get_trace_id(),
            "chat_id": get_chat_id(),
            "user_id": get_user_id(),
            "message_id": get_message_id(),
        }

        # Add exception info if present
        if record.exc_info:
            log_obj["exception"] = self.formatException(record.exc_info)

        return json.dumps(log_obj, ensure_ascii=False)


def remove_all_console_handlers():
    """
    Remove all console handlers (StreamHandler/RichHandler) from all loggers in the current process.
    """
    for _name, logger in logging.Logger.manager.loggerDict.items():
        if isinstance(logger, logging.Logger):
            handlers_to_remove = []
            for h in logger.handlers:
                if isinstance(h, (logging.StreamHandler, RichHandler)):
                    handlers_to_remove.append(h)
            for h in handlers_to_remove:
                logger.removeHandler(h)
                h.close()

    root_logger = logging.getLogger()
    handlers_to_remove = []
    for h in root_logger.handlers:
        if isinstance(h, logging.StreamHandler):
            handlers_to_remove.append(h)
    for h in handlers_to_remove:
        root_logger.removeHandler(h)
        h.close()


@contextmanager
def task_logging_context(task_id: str, log_dir: Path):
    token = TASK_CONTEXT_VAR.set(task_id)
    handler = make_task_logger(task_id, log_dir / "task_logs")
    try:
        yield
    finally:
        TASK_CONTEXT_VAR.reset(token)
        logging.getLogger().removeHandler(handler)
        handler.close()


def init_logging_for_benchmark_evaluation(print_task_logs=False):
    threading.Thread(target=start_zmq_listener, daemon=True).start()  # monitoring tool logs
    logging.basicConfig(handlers=[])
    setup_log_record_factory()
    if not print_task_logs:
        remove_all_console_handlers()


@lru_cache
def bootstrap_logger(
    level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] | int = "INFO",
    logger_name: str = "mem_deep_research",
    logger: logging.Logger | None = None,
    log_dir: str | Path | None = None,  # Log storage directory
    log_filename: str = "mem_deep_research.log",  # Default log filename
    to_console: bool = True,  # Whether to display to console
) -> logging.Logger:
    """Configure only this logger, not the root logger"""
    if logger is None:
        logger = logging.getLogger(logger_name)

    for handler in logger.handlers[:]:
        logger.removeHandler(handler)

    # Check if we should use JSON logging (production mode)
    is_prod = os.getenv("ENV") == "prod"

    if to_console:
        if is_prod:
            # Use JSON formatter in production
            handler = logging.StreamHandler()
            handler.setFormatter(JSONFormatter())
        else:
            # Use Rich handler in development
            handler = RichHandler(
                console=Console(
                    stderr=True,
                    width=200,
                    color_system=None,
                    force_terminal=False,
                    legacy_windows=False,
                ),
                rich_tracebacks=True,
                tracebacks_suppress=[hydra],
                tracebacks_show_locals=True,
                show_level=False,
            )
            formatter = logging.Formatter("[%(levelname)s] %(message)s")
            handler.setFormatter(formatter)
        logger.addHandler(handler)

    if log_dir is not None:
        log_dir = Path(log_dir)
        log_dir.mkdir(parents=True, exist_ok=True)
        file_path = log_dir / log_filename
        file_handler = logging.FileHandler(file_path, encoding="utf-8")
        if is_prod:
            # Use JSON formatter for file logging in production
            file_handler.setFormatter(JSONFormatter())
        else:
            # Use standard formatter in development
            file_handler.setFormatter(
                logging.Formatter("%(asctime)s [%(levelname)s] %(name)s: %(message)s")
            )
        logger.addHandler(file_handler)

    logger.setLevel(level)
    logger.propagate = True

    return logger



import dataclasses
import os

from mem_deep_research_core.llm.providers.openai_compatible_client import OpenAICompatibleClient
from mem_deep_research_core.mem_deep_research_logging.logger import (
    bootstrap_logger,
    truncate_for_log,
)

LOGGER_LEVEL = os.getenv("LOGGER_LEVEL", "INFO")
logger = bootstrap_logger(level=LOGGER_LEVEL)


@dataclasses.dataclass
class DeepSeekOpenRouterClient(OpenAICompatibleClient):
    def _get_api_credentials(self) -> tuple[str, str]:
        return self.cfg.llm.openrouter_api_key, self.cfg.llm.openrouter_base_url

    async def _create_message(
        self,
        system_prompt,
        messages,
        tools_definitions,
        keep_tool_result=-1,
        stream_message_callback=None,
    ):
        """Override to inject native tool_calls list into params via _customize_params."""
        # Pre-compute tool list and store for _customize_params to pick up
        self._pending_tool_list = await self.convert_tool_definition_to_tool_call(tools_definitions)
        return await super()._create_message(
            system_prompt, messages, tools_definitions, keep_tool_result, stream_message_callback
        )

    def _customize_params(self, params: dict) -> dict:
        params["stream"] = False
        if hasattr(self, "_pending_tool_list") and self._pending_tool_list:
            params["tools"] = self._pending_tool_list
        return params

    def process_llm_response(
        self, llm_response, message_history, agent_type="main"
    ) -> tuple[str, bool]:
        """Process OpenAI LLM response - extended with tool_calls finish_reason support."""
        if not llm_response or not llm_response.choices:
            error_msg = "LLM did not return a valid response."
            logger.error(f"Should never happen: {error_msg}")
            return "", True

        finish_reason = llm_response.choices[0].finish_reason

        if finish_reason == "tool_calls":
            tool_calls = llm_response.choices[0].message.tool_calls
            assistant_response_text = llm_response.choices[0].message.content or ""

            if not assistant_response_text:
                tool_call_descriptions = []
                for tool_call in tool_calls:
                    tool_call_descriptions.append(
                        f"Using tool {tool_call.function.name} with arguments: {tool_call.function.arguments}"
                    )
                assistant_response_text = "\n".join(tool_call_descriptions)

            message_history.append(
                {
                    "role": "assistant",
                    "content": assistant_response_text,
                    "tool_calls": [
                        {
                            "id": tc.id,
                            "type": "function",
                            "function": {
                                "name": tc.function.name,
                                "arguments": tc.function.arguments,
                            },
                        }
                        for tc in tool_calls
                    ],
                }
            )
            logger.debug(f"LLM Response: {truncate_for_log(assistant_response_text)}")
            return assistant_response_text, False

        # Delegate to base class for stop/length/other
        return super().process_llm_response(llm_response, message_history, agent_type)

    def extract_tool_calls_info(self, llm_response, assistant_response_text):
        """Extract tool call information - from response object for tool_calls finish_reason."""
        from mem_deep_research_core.utils.parsing_utils import parse_llm_response_for_tool_calls

        if llm_response.choices[0].finish_reason == "tool_calls":
            return parse_llm_response_for_tool_calls(llm_response.choices[0].message.tool_calls)
        else:
            return [], []

    def handle_max_turns_reached_summary_prompt(self, message_history, summary_prompt):
        """Handle max turns reached summary prompt - merge with last user message."""
        if message_history[-1]["role"] == "user":
            last_user_message = message_history.pop()
            return (
                last_user_message["content"][0]["text"]
                + "\n\n-----------------\n\n"
                + summary_prompt
            )
        else:
            return summary_prompt
